@@ -606,8 +606,8 @@ static t_account * accountlist_add_account(t_account * account)
         return NULL;
     }
     if (uid<1) {
-	eventlog(eventlog_level_error,__FUNCTION__,"got bad account (bad uid), fix it!");
-	uid = maxuserid + 1;
+    	ERROR2("got bad account (bad uid: %u) for \"%s\", fix it!", uid, username);
+	return NULL;
     }
 
     /* check whether the account limit was reached */
