@@ -206,6 +206,9 @@ extern char const * game_status_get_str(t_game_status status)
 	
     case game_status_done:
 	return "done";
+
+    case game_status_loaded:
+	return "loaded";
 	
     default:
 	return "UNKNOWN";
@@ -1380,7 +1383,7 @@ extern void game_set_status(t_game * game, t_game_status status)
 	// [quetzal] 20020829 - this should prevent invalid status changes
 	// its like started game cant become open and so on
 	if (game->status == game_status_started && 
-		(status == game_status_open || status == game_status_full)) {
+		(status == game_status_open || status == game_status_full || status == game_status_loaded)) {
 		eventlog(eventlog_level_error, "game_set_status", 
 		"attempting to set status '%s' (%d) to started game", game_status_get_str(status), status);
 		return;
