@@ -221,16 +221,18 @@ static int _client_anongame_profile(t_connection * c, t_packet const * const pac
 	int ffalevel=account_get_ladder_level(account,ctag,ladder_id_ffa);
 	int ffarank=account_get_ladder_rank(account,ctag,ladder_id_ffa);
 
-	int humanwins=account_get_racewins(account,1,ctag);
-	int humanlosses=account_get_racelosses(account,1,ctag);
-	int orcwins=account_get_racewins(account,2,ctag);
-	int orclosses=account_get_racelosses(account,2,ctag);
-	int undeadwins=account_get_racewins(account,8,ctag);
-	int undeadlosses=account_get_racelosses(account,8,ctag);
-	int nightelfwins=account_get_racewins(account,4,ctag);
-	int nightelflosses=account_get_racelosses(account,4,ctag);
-	int randomwins=account_get_racewins(account,0,ctag);
-	int randomlosses=account_get_racelosses(account,0,ctag);
+	int humanwins=account_get_racewins(account,W3_RACE_HUMANS,ctag);
+	int humanlosses=account_get_racelosses(account,W3_RACE_HUMANS,ctag);
+	int orcwins=account_get_racewins(account,W3_RACE_ORCS,ctag);
+	int orclosses=account_get_racelosses(account,W3_RACE_ORCS,ctag);
+	int undeadwins=account_get_racewins(account,W3_RACE_UNDEAD,ctag);
+	int undeadlosses=account_get_racelosses(account,W3_RACE_UNDEAD,ctag);
+	int nightelfwins=account_get_racewins(account,W3_RACE_NIGHTELVES,ctag);
+	int nightelflosses=account_get_racelosses(account,W3_RACE_NIGHTELVES,ctag);
+	int randomwins=account_get_racewins(account,W3_RACE_RANDOM,ctag);
+	int randomlosses=account_get_racelosses(account,W3_RACE_RANDOM,ctag);
+	int tourneywins=account_get_racewins(account,W3_RACE_DEMONS,ctag);
+	int tourneylosses=account_get_racelosses(account,W3_RACE_DEMONS,ctag);
 
 	unsigned char rescount;
 
@@ -326,8 +328,10 @@ static int _client_anongame_profile(t_connection * c, t_packet const * const pac
 	packet_append_data(rpacket,&temp,2); //elf wins
 	bn_int_set((bn_int*)&temp,nightelflosses);
 	packet_append_data(rpacket,&temp,2); //elf losses
-	temp=0;
-	packet_append_data(rpacket,&temp,4);
+	bn_int_set((bn_int*)&temp,tourneywins);
+	packet_append_data(rpacket,&temp,2); //tourney wins
+	bn_int_set((bn_int*)&temp,tourneylosses);
+	packet_append_data(rpacket,&temp,2); //tourney losses
 	//end of normal stats - Start of AT stats
 
 	/* 1 byte team count place holder, set later */
@@ -462,7 +466,7 @@ static int _client_anongame_get_icon(t_connection * c, t_packet const * const pa
 	// FIXED BY DJP 07/16/2003 FOR 110 CHANGE ( TOURNEY & RACE WINS ) + Table_witdh
 	short icon_req_race_wins;
 	short icon_req_tourney_wins;
-        int race[]={W3_RACE_RANDOM,W3_RACE_HUMANS,W3_RACE_ORCS,W3_RACE_UNDEAD,W3_RACE_NIGHTELVES,W3_ICON_DEMONS};
+        int race[]={W3_RACE_RANDOM,W3_RACE_HUMANS,W3_RACE_ORCS,W3_RACE_UNDEAD,W3_RACE_NIGHTELVES,W3_RACE_DEMONS};
         char race_char[6] ={'R','H','O','U','N','D'};
         char icon_pos[5] ={'2','3','4','5','6',};
         char table_width = 6;
@@ -644,7 +648,7 @@ static int _client_anongame_infos(t_connection * c, t_packet const * const packe
 		    packet_append_data(rpacket, tmpdata, tmplen);
 		    noitems++;
 		    server_tag_count++;
-		    eventlog(eventlog_level_debug,__FUNCTION__,"client_tag version=(0x%01x) request tagid=(0x%01x) tag=(%s)  tag_unk=(0x%04x)",conn_get_versionid(c),i,"CLIENT_FINDANONGAME_INFOTAG_URL",client_tag_unk);
+		    eventlog(eventlog_level_debug,__FUNCTION__,"client_tag request tagid=(0x%01x) tag=(%s)  tag_unk=(0x%04x)",i,"CLIENT_FINDANONGAME_INFOTAG_URL",client_tag_unk);
 		    break;
 		case CLIENT_FINDANONGAME_INFOTAG_MAP:
 		    bn_int_set((bn_int*)&server_tag_unk,0x70E2E0D5);
