@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Mihai RUSU (dizzy@roedu.net)
+ * Copyright (C) 2003 Dizzy 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,10 +19,8 @@
 #ifndef __HANDLERS_H__
 #define __HANDLERS_H__
 
-#include "common/setup_before.h"
 #include "common/packet.h"
 #include "connection.h"
-#include "common/setup_after.h"
 
 typedef int (* t_handler)(t_connection *, t_packet const * const);
 
