@@ -143,7 +143,6 @@ extern int ladderlist_make_all_active(void)
 extern int ladder_init_account(t_account * account, t_clienttag clienttag, t_ladder_id id)
 {
     int uid;
-    char clienttag_str[5];
     
     if (!account)
     {
@@ -161,7 +160,7 @@ extern int ladder_init_account(t_account * account, t_clienttag clienttag, t_lad
 	if (account_get_ladder_wins(account,clienttag,id)+
 	    account_get_ladder_losses(account,clienttag,id)>0) /* no ladder games so far... */
 	{
-	    eventlog(eventlog_level_warn,__FUNCTION__,"account for \"%s\" (%s) has %u wins and %u losses but has zero rating",account_get_name(account),clienttag,account_get_ladder_wins(account,clienttag,id),account_get_ladder_losses(account,clienttag,id));
+	    eventlog(eventlog_level_warn,__FUNCTION__,"account for \"%s\" (%s) has %u wins and %u losses but has zero rating",account_get_name(account),clienttag_uint_to_str(clienttag),account_get_ladder_wins(account,clienttag,id),account_get_ladder_losses(account,clienttag,id));
 	    return -1;
 	}
 	account_adjust_ladder_rating(account,clienttag,id,prefs_get_ladder_init_rating());
@@ -172,7 +171,7 @@ extern int ladder_init_account(t_account * account, t_clienttag clienttag, t_lad
 	war3_ladder_add(ladder_cw(clienttag,id),uid,0,0,account,0,clienttag);
 	war3_ladder_add(ladder_cg(clienttag,id),uid,0,0,account,0,clienttag);
 
-	eventlog(eventlog_level_info,__FUNCTION__,"initialized account for \"%s\" for \"%s\" ladder",account_get_name(account),tag_uint_to_str(clienttag_str,clienttag));
+	eventlog(eventlog_level_info,__FUNCTION__,"initialized account for \"%s\" for \"%s\" ladder",account_get_name(account),clienttag_uint_to_str(clienttag));
     }
 
     return 0;
@@ -253,12 +252,12 @@ extern int ladder_update(t_clienttag clienttag, t_ladder_id id, unsigned int cou
 	    return -1;
 	}
 	
-	return 0; /* no change in case of draw */
+	return -1; /* no change in case of draw */
     }
     if ((losers<1) || (winners<1) || (winners>1 && (winners!=losers)))
     {
 	eventlog(eventlog_level_info,__FUNCTION__,"missing winner or loser for count=%u (winners=%u losers=%u draws=%u) - discarding result",count,winners,losers,draws);
-	return 0;
+	return -1;
     }
     
     if (ladder_calc_info(clienttag,id,count,players,results,info)<0)
