@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2001  Dizzy (dizzy@roedu.net)
+ * Copyright (C) 2001  Dizzy 
+ * Copyright (C) 2004  Donny Redmond (dredmond@linuxmail.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -76,6 +77,7 @@
 #include "connection.h"
 #include "common/util.h"
 #include "common/eventlog.h"
+#include "common/xalloc.h"
 #include "account.h"
 #include "prefs.h"
 #include "mail.h"
@@ -91,7 +93,7 @@ static int get_mail_quota(t_account *);
 
 /* Mail API */
 /* for now this functions are only for internal use */
-static t_mailbox * mailbox_open(t_account *);
+static t_mailbox * mailbox_open(t_account *, t_mbox_mode mode);
 static int mailbox_count(t_mailbox *);
 static int mailbox_deliver(t_mailbox *, const char *, const char *);
 static t_mail * mailbox_read(t_mailbox *, unsigned int);
@@ -105,33 +107,34 @@ static void mailbox_close(t_mailbox *);
 static char * clean_str(char *);
 
 
-static t_mailbox * mailbox_open(t_account * user) {
+static t_mailbox * mailbox_open(t_account * user, t_mbox_mode mode) {
    t_mailbox * rez;
    char * path;
    char const * maildir;
 
-   if ((rez=malloc(sizeof(t_mailbox)))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_open","not enough memory");
-      return NULL;
-   }
+   rez=xmalloc(sizeof(t_mailbox));
+
    maildir=prefs_get_maildir();
-   p_mkdir(maildir,S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-   if ((path=malloc(strlen(maildir)+1+8+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_open","not enough memory");
-      free(rez);
-      return NULL;
-   }
+   if (mode & mbox_mode_write)
+      p_mkdir(maildir,S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+
+   path=xmalloc(strlen(maildir)+1+8+1);
    if (maildir[0]!='\0' && maildir[strlen(maildir)-1]=='/')
       sprintf(path,"%s%06u",maildir,account_get_uid(user));
    else
       sprintf(path,"%s/%06u",maildir,account_get_uid(user));
-   p_mkdir(path,S_IRWXU | S_IXGRP | S_IRGRP | S_IROTH | S_IXOTH);
+
+   if (mode & mbox_mode_write)
+      p_mkdir(path,S_IRWXU | S_IXGRP | S_IRGRP | S_IROTH | S_IXOTH);
+
    if ((rez->maildir=p_opendir(path))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_open","error opening maildir");
-      free(path);
-      free(rez);
+      if (mode & mbox_mode_write) 
+         eventlog(eventlog_level_error,__FUNCTION__,"error opening maildir");
+      xfree(path);
+      xfree(rez);
       return NULL;
    }
+
    rez->uid=account_get_uid(user);
    rez->path=path;
    return rez;
@@ -140,13 +143,12 @@ static t_mailbox * mailbox_open(t_account * user) {
 static int mailbox_count(t_mailbox *mailbox) {
    char const * dentry;
    int count=0;
-   
-   if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_count","got NULL mailbox");
-      return -1;
-   }
+
+   /* consider NULL mailbox as empty mailbox */
+   if (mailbox==NULL) return 0;
+
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_count","got NULL maildir");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return -1;
    }
    p_rewinddir(mailbox->maildir);
@@ -159,31 +161,28 @@ static int mailbox_deliver(t_mailbox * mailbox, const char * sender, const char
    char * filename;
 
    if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_deliver","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return -1;
    }
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_deliver","got NULL maildir");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return -1;
    }
    if (mailbox->path==NULL) {
-      eventlog(eventlog_level_error,"mailbox_deliver","got NULL path");
-      return -1;
-   }
-   if ((filename=malloc(strlen(mailbox->path)+1+15+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_deliver","not enough memory for filename");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL path");
       return -1;
    }
+   filename=xmalloc(strlen(mailbox->path)+1+15+1);
    sprintf(filename,"%s/%015lu",mailbox->path,(unsigned long)time(NULL));
    if ((fd=fopen(filename,"wb"))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_deliver","got NULL file descriptor. check permissions");
-      free(filename);
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL file descriptor. check permissions");
+      xfree(filename);
       return -1;
    }
    fprintf(fd,"%s\n",sender); /* write the sender on the first line of message */
    fprintf(fd,"%s\n",message); /* then write the actual message */
    fclose(fd);
-   free(filename);
+   xfree(filename);
    return 0;
 }
 
@@ -195,55 +194,37 @@ static t_mail * mailbox_read(t_mailbox * mailbox, unsigned int idx) {
    char *       filename;
    
    if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return NULL;
    }
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","got NULL maildir");
-      return NULL;
-   }
-   if ((rez=malloc(sizeof(t_mail)))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","not enough memory for message");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return NULL;
    }
+   rez=xmalloc(sizeof(t_mail));
    p_rewinddir(mailbox->maildir);
    dentry = NULL; /* if idx < 1 we should not crash :) */
    for(i=0;i<idx && (dentry=p_readdir(mailbox->maildir))!=NULL;i++)
      if (dentry[0]=='.') i--;
    if (dentry==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","index out of range");
-      free(rez);
+      eventlog(eventlog_level_error,__FUNCTION__,"index out of range");
+      xfree(rez);
       return NULL;
    }
    rez->timestamp=atoi(dentry);
-   if ((filename=malloc(strlen(dentry)+1+strlen(mailbox->path)+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","not enough memory for filename");
-      free(rez);
-      return NULL;
-   }
+   filename=xmalloc(strlen(dentry)+1+strlen(mailbox->path)+1);
    sprintf(filename,"%s/%s",mailbox->path,dentry);
    if ((fd=fopen(filename,"rb"))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","error while opening message");
-      free(rez);
-      free(filename);
-      return NULL;
-   }
-   free(filename);
-   if ((rez->sender=malloc(256))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","not enough memory for storing sender");
-      fclose(fd);
-      free(rez);
+      eventlog(eventlog_level_error,__FUNCTION__,"error while opening message");
+      xfree(rez);
+      xfree(filename);
       return NULL;
    }
+   xfree(filename);
+   rez->sender=xmalloc(256);
    fgets(rez->sender,256,fd); /* maybe 256 isnt the right value to bound a line but right now its all i have :) */
    clean_str(rez->sender);
-   if ((rez->message=malloc(256))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_read","not enough memory for storing message");
-      fclose(fd);
-      free(rez->sender);
-      free(rez);
-      return NULL;
-   }
+   rez->message=xmalloc(256);
    fgets(rez->message,256,fd);
    clean_str(rez->message);
    fclose(fd);
@@ -253,15 +234,15 @@ static t_mail * mailbox_read(t_mailbox * mailbox, unsigned int idx) {
 
 static void mailbox_unread(t_mail * mail) {
    if (mail==NULL)
-     eventlog(eventlog_level_error,"mailbox_unread","got NULL mail");
+     eventlog(eventlog_level_error,__FUNCTION__,"got NULL mail");
    else {
       if (mail->sender==NULL)
-	eventlog(eventlog_level_error,"mailbox_unread","got NULL sender");
-      else free(mail->sender);
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL sender");
+      else xfree(mail->sender);
       if (mail->message==NULL)
-	eventlog(eventlog_level_error,"mailbox_unread","got NULL message");
-      else free(mail->message);
-      free(mail);
+	eventlog(eventlog_level_error,__FUNCTION__,"got NULL message");
+      else xfree(mail->message);
+      xfree(mail);
    }
 }
 
@@ -272,39 +253,26 @@ static struct maillist_struct * mailbox_get_list(t_mailbox *mailbox) {
    char *sender,*filename;
    
    if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_get_list","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return NULL;
    }
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_get_list","got NULL maildir");
-      return NULL;
-   }
-   if ((filename=malloc(strlen(mailbox->path)+1+15+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_get_list","not enough memory for filename");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return NULL;
    }
+   filename=xmalloc(strlen(mailbox->path)+1+15+1);
    p_rewinddir(mailbox->maildir);
    for(;(dentry=p_readdir(mailbox->maildir))!=NULL;)
      if (dentry[0]!='.') {
-	q=malloc(sizeof(struct maillist_struct));
-	if (q==NULL) {
-	   eventlog(eventlog_level_error,"mailbox_get_list","not enough memory for list");
-	   free(filename);
-	   return rez;
-	}
+	q=xmalloc(sizeof(struct maillist_struct));
 	sprintf(filename,"%s/%s",mailbox->path,dentry);
 	if ((fd=fopen(filename,"rb"))==NULL) {
-	   eventlog(eventlog_level_error,"mailbox_get_list","error while opening message file");
-	   free(filename);
-	   free(q);
+	   eventlog(eventlog_level_error,__FUNCTION__,"error while opening message file");
+	   xfree(filename);
+	   xfree(q);
 	   return rez;
 	}
-	if ((sender=malloc(256))==NULL) {
-	   eventlog(eventlog_level_error,"mailbox_get_list","not enough memory for sender");
-	   fclose(fd);
-	   free(filename);
-	   free(q);
-	}
+	sender=xmalloc(256);
 	fgets(sender,256,fd);
 	clean_str(sender);
 	fclose(fd);
@@ -315,7 +283,7 @@ static struct maillist_struct * mailbox_get_list(t_mailbox *mailbox) {
 	else p->next=q;
 	p=q;
      }
-   free(filename);
+   xfree(filename);
    return rez;
 }
 
@@ -323,9 +291,9 @@ static void mailbox_unget_list(struct maillist_struct * maill) {
    struct maillist_struct *p, *q;
    
    for(p=maill;p!=NULL;p=q) {
-      if (p->sender!=NULL) free(p->sender);
+      if (p->sender!=NULL) xfree(p->sender);
       q=p->next;
-      free(p);
+      xfree(p);
    }
 }
 
@@ -336,11 +304,11 @@ static int mailbox_delete(t_mailbox * mailbox, unsigned int idx) {
    int          rez;
    
    if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return -1;
    }
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete","got NULL maildir");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return -1;
    }
    p_rewinddir(mailbox->maildir);
@@ -348,19 +316,16 @@ static int mailbox_delete(t_mailbox * mailbox, unsigned int idx) {
    for(i=0;i<idx && (dentry=p_readdir(mailbox->maildir))!=NULL;i++)
      if (dentry[0]=='.') i--;
    if (dentry==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete","index out of range");
-      return -1;
-   }
-   if ((filename=malloc(strlen(dentry)+1+strlen(mailbox->path)+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete","not enough memory for filename");
+      eventlog(eventlog_level_error,__FUNCTION__,"index out of range");
       return -1;
    }
+   filename=xmalloc(strlen(dentry)+1+strlen(mailbox->path)+1);
    sprintf(filename,"%s/%s",mailbox->path,dentry);
    rez=remove(filename);
    if (rez<0) {
-       eventlog(eventlog_level_info,"mailbox_delete","could not remove file \"%s\" (remove: %s)",filename,strerror(errno));
+       eventlog(eventlog_level_info,__FUNCTION__,"could not remove file \"%s\" (remove: %s)",filename,pstrerror(errno));
     }
-   free(filename);
+   xfree(filename);
    return rez;
 }
 
@@ -370,17 +335,14 @@ static int mailbox_delete_all(t_mailbox * mailbox) {
    int          count;
    
    if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete_all","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return -1;
    }
    if (mailbox->maildir==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete_all","got NULL maildir");
-      return -1;
-   }
-   if ((filename=malloc(strlen(mailbox->path)+1+15+1))==NULL) {
-      eventlog(eventlog_level_error,"mailbox_delete_all","not enough memory for filename");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
       return -1;
    }
+   filename=xmalloc(strlen(mailbox->path)+1+15+1);
    p_rewinddir(mailbox->maildir);
    count = 0;
    while ((dentry=p_readdir(mailbox->maildir))!=NULL)
@@ -388,22 +350,19 @@ static int mailbox_delete_all(t_mailbox * mailbox) {
 	sprintf(filename,"%s/%s",mailbox->path,dentry);
 	if (!remove(filename)) count++;
      }
-   free(filename);
+   xfree(filename);
    return count;
 }
 
 static void mailbox_close(t_mailbox *mailbox) {
-   if (mailbox==NULL) {
-      eventlog(eventlog_level_error,"mailbox_close","got NULL mailbox");
-      return;
-   }
+   if (mailbox==NULL) return;
    if (mailbox->maildir!=NULL) {
       p_closedir(mailbox->maildir);
    } else {
-      eventlog(eventlog_level_error,"mailbox_close","got NULL maildir");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL maildir");
    }
-   if (mailbox->path) free(mailbox->path);
-   free(mailbox);
+   if (mailbox->path) xfree(mailbox->path);
+   xfree(mailbox);
 }
 
 static char * clean_str(char * str) {
@@ -493,11 +452,11 @@ static void mail_func_send(t_connection * c, const char * str) {
    t_mailbox * mailbox;
    
    if (c==NULL) {
-      eventlog(eventlog_level_error,"mail_func_send","got NULL connection");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL connection");
       return;
    }
    if (str==NULL) {
-      eventlog(eventlog_level_error,"mail_func_send","got NULL command string");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL command string");
       return;
    }
    for(i=0;str[i]==' ';i++); /* skip any spaces */
@@ -513,19 +472,15 @@ static void mail_func_send(t_connection * c, const char * str) {
       message_send_text(c,message_type_error,c,"Syntax: /mail send <receiver> <message>");
       return;
    }
-   if ((dest=malloc(i+1))==NULL) {
-      eventlog(eventlog_level_error,"mail_func_send","not enough memory for receiver");
-      message_send_text(c,message_type_error,c,"Not enough resources to complete request!");
-      return;
-   }
+   dest=xmalloc(i+1);
    memmove(dest,p,i); dest[i]='\0'; /* copy receiver in his separate string */
    if ((recv=accountlist_find_account(dest))==NULL) { /* is dest a valid account on this server ? */
       message_send_text(c,message_type_error,c,"Receiver UNKNOWN!");
-      free(dest);
+      xfree(dest);
       return;
    }
-   free(dest); /* free dest here, the sooner the better */
-   if ((mailbox=mailbox_open(recv))==NULL) {
+   xfree(dest); /* free dest here, the sooner the better */
+   if ((mailbox=mailbox_open(recv, mbox_mode_write))==NULL) {
       message_send_text(c,message_type_error,c,"There was an error completing your request!");
       return;
    }
@@ -539,7 +494,6 @@ static void mail_func_send(t_connection * c, const char * str) {
      message_send_text(c,message_type_error,c,"There was an error completing your request!");
    else 
      message_send_text(c,message_type_info,c,"Your mail has been sent successfully.");
-   conn_unget_username(c,myname);
    mailbox_close(mailbox);
 }
 
@@ -551,34 +505,32 @@ static void mail_func_read(t_connection * c, const char * str) {
    int i;
    
    if (c==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL connection");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL connection");
       return;
    }
    if (str==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL command string");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL command string");
       return;
    }
    for(i=0;str[i]==' ';i++);
    p=str+i;
    if ((user=conn_get_account(c))==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL account");
-      return;
-   }
-   if ((mailbox=mailbox_open(user))==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL mailbox");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL account");
       return;
    }
+
+   mailbox=mailbox_open(user, mbox_mode_read);
    if (*p=='\0') { /* user wants to see the mail summary */
       struct maillist_struct *maill, *mp;
       unsigned int idx;
       
-      if (mailbox_count(mailbox)==0) {
+      if (!mailbox_count(mailbox)) {
 	 message_send_text(c,message_type_info,c,"You have no mail.");
 	 mailbox_close(mailbox);
 	 return;
       }
       if ((maill=mailbox_get_list(mailbox))==NULL) {
-	 eventlog(eventlog_level_error,"mail_func_read","got NULL maillist");
+	 eventlog(eventlog_level_error,__FUNCTION__,"got NULL maillist");
 	 mailbox_close(mailbox);
 	 return;
       }
@@ -631,11 +583,11 @@ static void mail_func_delete(t_connection * c, const char * str) {
    int i;
    
    if (c==NULL) {
-      eventlog(eventlog_level_error,"mail_func_delete","got NULL connection");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL connection");
       return;
    }
    if (str==NULL) {
-      eventlog(eventlog_level_error,"mail_func_delete","got NULL command string");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL command string");
       return;
    }
    for(i=0;str[i]==' ';i++);
@@ -645,11 +597,11 @@ static void mail_func_delete(t_connection * c, const char * str) {
       return;
    }
    if ((user=conn_get_account(c))==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL account");
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL account");
       return;
    }
-   if ((mailbox=mailbox_open(user))==NULL) {
-      eventlog(eventlog_level_error,"mail_func_read","got NULL mailbox");
+   if ((mailbox=mailbox_open(user, mbox_mode_write))==NULL) {
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL mailbox");
       return;
    }
    if (strcmp(p,"all")==0) {
@@ -689,7 +641,6 @@ static void mail_func_delete(t_connection * c, const char * str) {
    mailbox_close(mailbox);
 }
 
-
 static void mail_usage(t_connection * c) {
    message_send_text(c,message_type_info,c,"to print this information:");
    message_send_text(c,message_type_info,c,"    /mail help");
@@ -707,3 +658,32 @@ static void mail_usage(t_connection * c) {
    message_send_text(c,message_type_info,c,"    send: s");
    message_send_text(c,message_type_info,c,"    delete: del");
 }
+
+extern char const * check_mail(t_connection const * c) {
+   t_account * user;
+   t_mailbox * mailbox;
+   static char tmp[64];
+   int count;
+
+   if (!(c)) {
+      eventlog(eventlog_level_error,__FUNCTION__,"got NULL connection");
+      return "";
+   }
+
+   if (!(user=conn_get_account(c)))
+      return "";
+
+   mailbox=mailbox_open(user, mbox_mode_read);
+   count = mailbox_count(mailbox);
+   mailbox_close(mailbox);
+
+   if (count == 0) 
+   {
+      return "You have no mail.";
+   }
+   else
+   {
+      sprintf(tmp,"You have %d message(s) in your mailbox.",count);
+      return tmp;
+   }
+}
