@@ -53,6 +53,11 @@ typedef enum
     message_type_uniqueid,
     message_type_mode,
 
+    /**
+    *   IRC specific messages
+    */
+    message_type_notice,
+
     /**
     *  Westwood Online Extensions
     */
