@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1998,1999,2000,2001  Ross Combs (rocombs@cs.nmsu.edu)
  * Copyright (C) 2000,2001  Marco Ziech (mmz@gmx.net)
- * Copyright (C) 2002,2003 Mihai RUSU (dizzy@rdsnet.ro)
+ * Copyright (C) 2002,2003,2004 Dizzy 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,11 +66,9 @@
 #endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
-#else
-# ifdef WIN32
-#  include <io.h>
-# endif
 #endif
+#include "compat/access.h"
+#include "compat/rename.h"
 #include "compat/pdir.h"
 #include "common/eventlog.h"
 #include "prefs.h"
@@ -78,44 +76,56 @@
 #include "common/field_sizes.h"
 #include "common/bnethash.h"
 #define CLAN_INTERNAL_ACCESS
-#define ACCOUNT_INTERNAL_ACCESS
+#define TEAM_INTERNAL_ACCESS
 #include "common/introtate.h"
+#include "team.h"
 #include "account.h"
 #include "common/hashtable.h"
 #include "storage.h"
+#include "storage_file.h"
+#include "file_plain.h"
+#include "file_cdb.h"
 #include "common/list.h"
 #include "connection.h"
 #include "watch.h"
 #include "clan.h"
 #undef ACCOUNT_INTERNAL_ACCESS
+#undef TEAM_INTERNAL_ACCESS
 #undef CLAN_INTERNAL_ACCESS
 #include "common/tag.h"
+#include "common/xalloc.h"
+#include "common/elist.h"
 #include "common/setup_after.h"
 
 /* file storage API functions */
 
 static int file_init(const char *);
 static int file_close(void);
+static unsigned file_read_maxuserid(void);
 static t_storage_info *file_create_account(char const *);
 static t_storage_info *file_get_defacct(void);
 static int file_free_info(t_storage_info *);
 static int file_read_attrs(t_storage_info *, t_read_attr_func, void *);
-static void *file_read_attr(t_storage_info *, const char *);
-static int file_write_attrs(t_storage_info *, void *);
-static int file_read_accounts(t_read_accounts_func, void *);
-static void *file_read_account(t_read_account_func, const char *);
+static t_attr *file_read_attr(t_storage_info *, const char *);
+static int file_write_attrs(t_storage_info *, const t_hlist *);
+static int file_read_accounts(int,t_read_accounts_func, void *);
+static t_storage_info *file_read_account(const char *, unsigned);
 static int file_cmp_info(t_storage_info *, t_storage_info *);
 static const char *file_escape_key(const char *);
 static int file_load_clans(t_load_clans_func);
 static int file_write_clan(void *);
 static int file_remove_clan(int);
 static int file_remove_clanmember(int);
+static int file_load_teams(t_load_teams_func);
+static int file_write_team(void *);
+static int file_remove_team(unsigned int);
 
 /* storage struct populated with the functions above */
 
 t_storage storage_file = {
     file_init,
     file_close,
+    file_read_maxuserid,
     file_create_account,
     file_get_defacct,
     file_free_info,
@@ -129,21 +139,33 @@ t_storage storage_file = {
     file_load_clans,
     file_write_clan,
     file_remove_clan,
-    file_remove_clanmember
+    file_remove_clanmember,
+    file_load_teams,
+    file_write_team,
+    file_remove_team
 };
 
 /* start of actual file storage code */
 
 static const char *accountsdir = NULL;
 static const char *clansdir = NULL;
+static const char *teamsdir = NULL;
 static const char *defacct = NULL;
+static t_file_engine *file = NULL;
+
+static unsigned file_read_maxuserid(void)
+{
+    return maxuserid;
+}
 
 static int file_init(const char *path)
 {
     char *tok, *copy, *tmp, *p;
     const char *dir = NULL;
     const char *clan = NULL;
+    const char *team = NULL;
     const char *def = NULL;
+    const char *driver = NULL;
 
     if (path == NULL || path[0] == '\0')
     {
@@ -151,12 +173,7 @@ static int file_init(const char *path)
 	return -1;
     }
 
-    if ((copy = strdup(path)) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not duplicate path");
-	return -1;
-    }
-
+    copy = xstrdup(path);
     tmp = copy;
     while ((tok = strtok(tmp, ";")) != NULL)
     {
@@ -164,7 +181,7 @@ static int file_init(const char *path)
 	if ((p = strchr(tok, '=')) == NULL)
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid storage_path, no '=' present in token");
-	    free((void *) copy);
+	    xfree((void *) copy);
 	    return -1;
 	}
 	*p = '\0';
@@ -172,46 +189,43 @@ static int file_init(const char *path)
 	    dir = p + 1;
 	else if (strcasecmp(tok, "clan") == 0)
 	    clan = p + 1;
+	else if (strcasecmp(tok, "team") == 0)
+	    team = p + 1;
 	else if (strcasecmp(tok, "default") == 0)
 	    def = p + 1;
+	else if (strcasecmp(tok, "mode") == 0)
+	    driver = p + 1;
 	else
 	    eventlog(eventlog_level_warn, __FUNCTION__, "unknown token in storage_path : '%s'", tok);
     }
 
-    if (def == NULL || clan == NULL || dir == NULL)
+    if (def == NULL || clan == NULL || team == NULL || dir == NULL || driver == NULL)
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "invalid storage_path line for file module (doesnt have a 'dir', a 'clan' and a 'default' token)");
-	free((void *) copy);
+	eventlog(eventlog_level_error, __FUNCTION__, "invalid storage_path line for file module (doesnt have a 'dir', a 'clan', a 'team', a 'default' token and a 'mode' token)");
+	xfree((void *) copy);
+	return -1;
+    }
+
+    if (!strcasecmp(driver, "plain"))
+	file = &file_plain;
+    else if (!strcasecmp(driver, "cdb"))
+	file = &file_cdb;
+    else
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "unknown mode '%s' must be either plain or cdb", driver);
+	xfree((void *) copy);
 	return -1;
     }
 
     if (accountsdir)
 	file_close();
 
-    if ((accountsdir = strdup(dir)) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to store accounts dir");
-	free((void *) copy);
-	return -1;
-    }
+    accountsdir = xstrdup(dir);
+    clansdir = xstrdup(clan);
+    teamsdir = xstrdup(team);
+    defacct = xstrdup(def);
 
-    if ((clansdir = strdup(clan)) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to store clans dir");
-	free((void *) copy);
-	return -1;
-    }
-
-    if ((defacct = strdup(def)) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to store default account path");
-	free((void *) accountsdir);
-	accountsdir = NULL;
-	free((void *) copy);
-	return -1;
-    }
-
-    free((void *) copy);
+    xfree((void *) copy);
 
     return 0;
 }
@@ -219,17 +233,23 @@ static int file_init(const char *path)
 static int file_close(void)
 {
     if (accountsdir)
-	free((void *) accountsdir);
+	xfree((void *) accountsdir);
     accountsdir = NULL;
 
     if (clansdir)
-	free((void *) clansdir);
+	xfree((void *) clansdir);
     clansdir = NULL;
 
+    if (teamsdir)
+    	xfree((void *) teamsdir);
+    teamsdir = NULL;
+
     if (defacct)
-	free((void *) defacct);
+	xfree((void *) defacct);
     defacct = NULL;
 
+    file = NULL;
+
     return 0;
 }
 
@@ -237,7 +257,7 @@ static t_storage_info *file_create_account(const char *username)
 {
     char *temp;
 
-    if (accountsdir == NULL)
+    if (accountsdir == NULL || file == NULL)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "file storage not initilized");
 	return NULL;
@@ -247,41 +267,34 @@ static t_storage_info *file_create_account(const char *username)
     {
 	char const *safename;
 
+	if (!strcmp(username, defacct))
+	{
+	    eventlog(eventlog_level_error, __FUNCTION__, "username as defacct not allowed");
+	    return NULL;
+	}
+
 	if (!(safename = escape_fs_chars(username, strlen(username))))
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "could not escape username");
 	    return NULL;
 	}
-	if (!(temp = malloc(strlen(accountsdir) + 1 + strlen(safename) + 1)))	/* dir + / + name + NUL */
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for temp");
-	    free((void *) safename);
-	    return NULL;
-	}
+	temp = xmalloc(strlen(accountsdir) + 1 + strlen(safename) + 1);	/* dir + / + name + NUL */
 	sprintf(temp, "%s/%s", accountsdir, safename);
-	free((void *) safename);	/* avoid warning */
+	xfree((void *) safename);	/* avoid warning */
     } else
     {
-	if (!(temp = malloc(strlen(accountsdir) + 1 + 8 + 1)))	/* dir + / + uid + NUL */
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for temp");
-	    return NULL;
-	}
+	temp = xmalloc(strlen(accountsdir) + 1 + 8 + 1);	/* dir + / + uid + NUL */
 	sprintf(temp, "%s/%06u", accountsdir, maxuserid + 1);	/* FIXME: hmm, maybe up the %06 to %08... */
     }
 
     return temp;
 }
 
-static int file_write_attrs(t_storage_info * info, void *attributes)
+static int file_write_attrs(t_storage_info * info, const t_hlist *attributes)
 {
-    FILE *accountfile;
-    t_attribute *attr;
-    char const *key;
-    char const *val;
     char *tempname;
 
-    if (accountsdir == NULL)
+    if (accountsdir == NULL || file == NULL)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "file storage not initilized");
 	return -1;
@@ -299,108 +312,31 @@ static int file_write_attrs(t_storage_info * info, void *attributes)
 	return -1;
     }
 
-    if (!(tempname = malloc(strlen(accountsdir) + 1 + strlen(BNETD_ACCOUNT_TMP) + 1)))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to allocate memory for tempname");
-	return -1;
-    }
-
+    tempname = xmalloc(strlen(accountsdir) + 1 + strlen(BNETD_ACCOUNT_TMP) + 1);
     sprintf(tempname, "%s/%s", accountsdir, BNETD_ACCOUNT_TMP);
 
-    if (!(accountfile = fopen(tempname, "w")))
+    if (file->write_attrs(tempname, attributes))
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to open file \"%s\" for writing (fopen: %s)", tempname, strerror(errno));
-	free(tempname);
+	/* no eventlog here, it should be reported from the file layer */
+	xfree(tempname);
 	return -1;
     }
 
-    for (attr = (t_attribute *) attributes; attr; attr = attr->next)
+    if (p_rename(tempname, (const char *) info) < 0)
     {
-	if (attr->key)
-	    key = escape_chars(attr->key, strlen(attr->key));
-	else
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "attribute with NULL key in list");
-	    key = NULL;
-	}
-
-	if (attr->val)
-	    val = escape_chars(attr->val, strlen(attr->val));
-	else
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "attribute with NULL val in list");
-	    val = NULL;
-	}
-
-	if (key && val)
-	{
-	    if (strncmp("BNET\\CharacterDefault\\", key, 20) == 0)
-	    {
-		eventlog(eventlog_level_debug, __FUNCTION__, "skipping attribute key=\"%s\"", attr->key);
-	    } else
-	    {
-		eventlog(eventlog_level_debug, __FUNCTION__, "saving attribute key=\"%s\" val=\"%s\"", attr->key, attr->val);
-		fprintf(accountfile, "\"%s\"=\"%s\"\n", key, val);
-	    }
-	} else
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not save attribute key=\"%s\"", attr->key);
-
-	if (key)
-	    free((void *) key);	/* avoid warning */
-	if (val)
-	    free((void *) val);	/* avoid warning */
-    }
-
-    if (fclose(accountfile) < 0)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not close account file \"%s\" after writing (fclose: %s)", tempname, strerror(errno));
-	free(tempname);
-	return -1;
-    }
-#ifdef WIN32
-    /* We are about to rename the temporary file
-     * to replace the existing account.  In Windows,
-     * we have to remove the previous file or the
-     * rename function will fail saying the file
-     * already exists.  This defeats the purpose of
-     * the rename which was to make this an atomic
-     * operation.  At least the race window is small.
-     */
-    if (access((const char *) info, 0) == 0)
-    {
-	if (remove((const char *) info) < 0)
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not delete account file \"%s\" (remove: %s)", (char *) info, strerror(errno));
-	    free(tempname);
-	    return -1;
-	}
-    }
-#endif
-
-    if (rename(tempname, (const char *) info) < 0)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not rename account file to \"%s\" (rename: %s)", (char *) info, strerror(errno));
-	free(tempname);
+	eventlog(eventlog_level_error, __FUNCTION__, "could not rename account file to \"%s\" (rename: %s)", (char *) info, pstrerror(errno));
+	xfree(tempname);
 	return -1;
     }
 
-    free(tempname);
+    xfree(tempname);
 
     return 0;
 }
 
 static int file_read_attrs(t_storage_info * info, t_read_attr_func cb, void *data)
 {
-    FILE *accountfile;
-    unsigned int line;
-    char const *buff;
-    unsigned int len;
-    char *esckey;
-    char *escval;
-    char *key;
-    char *val;
-
-    if (accountsdir == NULL)
+    if (accountsdir == NULL || file == NULL)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "file storage not initilized");
 	return -1;
@@ -419,89 +355,43 @@ static int file_read_attrs(t_storage_info * info, t_read_attr_func cb, void *dat
     }
 
     eventlog(eventlog_level_debug, __FUNCTION__, "loading \"%s\"", (char *) info);
-    if (!(accountfile = fopen((const char *) info, "r")))
+
+    if (file->read_attrs((const char *) info, cb, data))
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not open account file \"%s\" for reading (fopen: %s)", (char *) info, strerror(errno));
+	/* no eventlog, error reported earlier */
 	return -1;
     }
 
-    for (line = 1; (buff = file_get_line(accountfile)); line++)
-    {
-	if (buff[0] == '#' || buff[0] == '\0')
-	{
-	    free((void *) buff);	/* avoid warning */
-	    continue;
-	}
-
-	if (strlen(buff) < 6)	/* "?"="" */
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "malformed line %d of account file \"%s\"", line, (char *) info);
-	    free((void *) buff);	/* avoid warning */
-	    continue;
-	}
-
-	len = strlen(buff) - 5 + 1;	/* - ""="" + NUL */
-	if (!(esckey = malloc(len)))
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for esckey on line %d of account file \"%s\"", line, (char *) info);
-	    free((void *) buff);	/* avoid warning */
-	    continue;
-	}
-	if (!(escval = malloc(len)))
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for escval on line %d of account file \"%s\"", line, (char *) info);
-	    free((void *) buff);	/* avoid warning */
-	    free(esckey);
-	    continue;
-	}
-
-	if (sscanf(buff, "\"%[^\"]\" = \"%[^\"]\"", esckey, escval) != 2)
-	{
-	    if (sscanf(buff, "\"%[^\"]\" = \"\"", esckey) != 1)	/* hack for an empty value field */
-	    {
-		eventlog(eventlog_level_error, __FUNCTION__, "malformed entry on line %d of account file \"%s\"", line, (char *) info);
-		free(escval);
-		free(esckey);
-		free((void *) buff);	/* avoid warning */
-		continue;
-	    }
-	    escval[0] = '\0';
-	}
-	free((void *) buff);	/* avoid warning */
-
-	key = unescape_chars(esckey);
-	val = unescape_chars(escval);
-
-/* eventlog(eventlog_level_debug,"account_load_attrs","strlen(esckey)=%u (%c), len=%u",strlen(esckey),esckey[0],len);*/
-	free(esckey);
-	free(escval);
-
-	if (cb(key, val, data))
-	    eventlog(eventlog_level_error, __FUNCTION__, "got error from callback (key: '%s' val:'%s')", key, val);
-
-	if (key)
-	    free((void *) key);	/* avoid warning */
-	if (val)
-	    free((void *) val);	/* avoid warning */
-    }
-
-
-    if (fclose(accountfile) < 0)
-	eventlog(eventlog_level_error, __FUNCTION__, "could not close account file \"%s\" after reading (fclose: %s)", (char *) info, strerror(errno));
-
     return 0;
 }
 
-static void *file_read_attr(t_storage_info * info, const char *key)
+static t_attr *file_read_attr(t_storage_info * info, const char *key)
 {
-    /* flat file storage doesnt know to read selective attributes */
-    return NULL;
+    if (accountsdir == NULL || file == NULL)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "file storage not initilized");
+	return NULL;
+    }
+
+    if (info == NULL)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "got NULL info storage");
+	return NULL;
+    }
+
+    if (key == NULL)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "got NULL key");
+	return NULL;
+    }
+
+    return file->read_attr((const char *) info, key);
 }
 
 static int file_free_info(t_storage_info * info)
 {
     if (info)
-	free((void *) info);
+	xfree((void *) info);
     return 0;
 }
 
@@ -515,16 +405,12 @@ static t_storage_info *file_get_defacct(void)
 	return NULL;
     }
 
-    if ((info = strdup(defacct)) == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not duplicate default account filename");
-	return NULL;
-    }
+    info = xstrdup(defacct);
 
     return info;
 }
 
-static int file_read_accounts(t_read_accounts_func cb, void *data)
+static int file_read_accounts(int flag,t_read_accounts_func cb, void *data)
 {
     char const *dentry;
     char *pathname;
@@ -544,7 +430,7 @@ static int file_read_accounts(t_read_accounts_func cb, void *data)
 
     if (!(accountdir = p_opendir(accountsdir)))
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to open user directory \"%s\" for reading (p_opendir: %s)", accountsdir, strerror(errno));
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to open user directory \"%s\" for reading (p_opendir: %s)", accountsdir, pstrerror(errno));
 	return -1;
     }
 
@@ -553,23 +439,19 @@ static int file_read_accounts(t_read_accounts_func cb, void *data)
 	if (dentry[0] == '.')
 	    continue;
 
-	if (!(pathname = malloc(strlen(accountsdir) + 1 + strlen(dentry) + 1)))	/* dir + / + file + NUL */
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for pathname");
-	    continue;
-	}
+	pathname = xmalloc(strlen(accountsdir) + 1 + strlen(dentry) + 1);	/* dir + / + file + NUL */
 	sprintf(pathname, "%s/%s", accountsdir, dentry);
 
 	cb(pathname, data);
     }
 
     if (p_closedir(accountdir) < 0)
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to close user directory \"%s\" (p_closedir: %s)", accountsdir, strerror(errno));
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to close user directory \"%s\" (p_closedir: %s)", accountsdir, pstrerror(errno));
 
     return 0;
 }
 
-static void *file_read_account(t_read_account_func cb, const char *accname)
+static t_storage_info *file_read_account(const char *accname, unsigned uid)
 {
     char *pathname;
 
@@ -579,25 +461,21 @@ static void *file_read_account(t_read_account_func cb, const char *accname)
 	return NULL;
     }
 
-    if (cb == NULL)
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "got NULL callback");
-	return NULL;
+    /* ONLY if requesting for a username and if savebyname() is true
+     * PS: yes its kind of a hack, we will make a proper index file
+     */
+    if (accname && prefs_get_savebyname()) {
+	pathname = xmalloc(strlen(accountsdir) + 1 + strlen(accname) + 1);	/* dir + / + file + NUL */
+	sprintf(pathname, "%s/%s", accountsdir, accname);
+	if (access(pathname, F_OK))	/* if it doesn't exist */
+	{
+	    xfree((void *) pathname);
+	    return NULL;
+	}
+	return pathname;
     }
 
-    if (!(pathname = malloc(strlen(accountsdir) + 1 + strlen(accname) + 1)))	/* dir + / + file + NUL */
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for pathname");
-	return NULL;
-    }
-    sprintf(pathname, "%s/%s", accountsdir, accname);
-    if (access(pathname, 0))	/* if it doesn't exist */
-    {
-	free((void *) pathname);
-	return NULL;
-    }
-
-    return cb(pathname);
+    return NULL;
 }
 
 static int file_cmp_info(t_storage_info * info1, t_storage_info * info2)
@@ -633,17 +511,12 @@ static int file_load_clans(t_load_clans_func cb)
 
     if (!(clandir = p_opendir(clansdir)))
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to open clan directory \"%s\" for reading (p_opendir: %s)", clansdir, strerror(errno));
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to open clan directory \"%s\" for reading (p_opendir: %s)", clansdir, pstrerror(errno));
 	return -1;
     }
     eventlog(eventlog_level_trace, __FUNCTION__, "start reading clans");
 
-    if (!(pathname = malloc(strlen(clansdir) + 1 + 4 + 1)))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for pathname");
-	return -1;
-    }
-
+    pathname = xmalloc(strlen(clansdir) + 1 + 4 + 1);
     while ((dentry = p_readdir(clandir)) != NULL)
     {
 	if (dentry[0] == '.')
@@ -665,20 +538,13 @@ static int file_load_clans(t_load_clans_func cb)
 	    continue;
 	}
 
-	if (!(clan = malloc(sizeof(t_clan))))
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clan");
-	    free((void *) pathname);
-	    p_closedir(clandir);
-	    return -1;
-	}
-
+	clan = xmalloc(sizeof(t_clan));
 	clan->clantag = clantag;
 
 	if (!fgets(line, 1024, fp))
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: no first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 
@@ -686,7 +552,7 @@ static int file_load_clans(t_load_clans_func cb)
 	if (*clanname != '"')
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 	clanname++;
@@ -694,14 +560,14 @@ static int file_load_clans(t_load_clans_func cb)
 	if (!p)
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 	*p = '\0';
 	if (strlen(clanname) >= CLAN_NAME_MAX)
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 
@@ -709,14 +575,14 @@ static int file_load_clans(t_load_clans_func cb)
 	if (*p != ',')
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 	p++;
 	if (*p != '"')
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 	motd = p + 1;
@@ -724,7 +590,7 @@ static int file_load_clans(t_load_clans_func cb)
 	if (!p)
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid clan file: invalid first line");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
 	*p = '\0';
@@ -732,21 +598,11 @@ static int file_load_clans(t_load_clans_func cb)
 	if (sscanf(p + 1, ",%d,%d\n", &cid, &creation_time) != 2)
 	{
 	    eventlog(eventlog_level_error, __FUNCTION__, "invalid first line in clanfile");
-	    free((void*)clan);
-	    continue;
-	}
-	if ((clan->clanname = strdup(clanname)) == NULL)
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to store clanname");
-	    free((void*)clan);
-	    continue;
-	}
-	if ((clan->clan_motd = strdup(motd)) == NULL)
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "not enough memory to store motd");
-	    free((void*)clan);
+	    xfree((void*)clan);
 	    continue;
 	}
+	clan->clanname = xstrdup(clanname);
+	clan->clan_motd = xstrdup(motd);
 	clan->clanid = cid;
 	clan->creation_time = (time_t) creation_time;
 	clan->created = 1;
@@ -755,36 +611,17 @@ static int file_load_clans(t_load_clans_func cb)
 
 	eventlog(eventlog_level_trace, __FUNCTION__, "name: %s motd: %s clanid: %i time: %i", clanname, motd, cid, creation_time);
 
-	if ((clan->members = list_create()) == NULL)
-	{
-	    eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for clan members");
-	    free((void*)clan);
-	    free((void *) pathname);
-	    p_closedir(clandir);
-	    return -1;
-	}
+	clan->members = list_create();
 
 	while (fscanf(fp, "%i,%c,%i\n", &member_uid, &member_status, &member_join_time) == 3)
 	{
-	    if (!(member = malloc(sizeof(t_clanmember))))
-	    {
-		eventlog(eventlog_level_error, __FUNCTION__, "cannot allocate memory for clan member");
-		clan_remove_all_members(clan);
-		free((void *) clan->clanname);
-		free((void *) clan->clan_motd);
-		free((void *) clan);
-		fclose(fp);
-		free((void *) pathname);
-		p_closedir(clandir);
-		return -1;
-	    }
+	    member = xmalloc(sizeof(t_clanmember));
 	    if (!(member->memberacc = accountlist_find_account_by_uid(member_uid)))
 	    {
 		eventlog(eventlog_level_error, __FUNCTION__, "cannot find uid %u", member_uid);
-		free((void *) member);
+		xfree((void *) member);
 		continue;
 	    }
-	    member->memberconn = NULL;
 	    member->status = member_status - '0';
 	    member->join_time = member_join_time;
 	    member->clan = clan;
@@ -795,19 +632,8 @@ static int file_load_clans(t_load_clans_func cb)
 		clan->modified = 1;
 	    }
 
-	    if (list_append_data(clan->members, member) < 0)
-	    {
-		eventlog(eventlog_level_error, __FUNCTION__, "could not append item");
-		free((void *) member);
-		clan_remove_all_members(clan);
-		free((void *) clan->clanname);
-		free((void *) clan->clan_motd);
-		free((void *) clan);
-		fclose(fp);
-		free((void *) pathname);
-		p_closedir(clandir);
-		return -1;
-	    }
+	    list_append_data(clan->members, member);
+
 	    account_set_clanmember(member->memberacc, member);
 	    eventlog(eventlog_level_trace, __FUNCTION__, "added member: uid: %i status: %c join_time: %i", member_uid, member_status + '0', member_join_time);
 	}
@@ -818,11 +644,11 @@ static int file_load_clans(t_load_clans_func cb)
 
     }
 
-    free((void *) pathname);
+    xfree((void *) pathname);
 
     if (p_closedir(clandir) < 0)
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "unable to close clan directory \"%s\" (p_closedir: %s)", clansdir, strerror(errno));
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to close clan directory \"%s\" (p_closedir: %s)", clansdir, pstrerror(errno));
     }
     eventlog(eventlog_level_trace, __FUNCTION__, "finished reading clans");
 
@@ -836,18 +662,14 @@ static int file_write_clan(void *data)
     t_clanmember *member;
     char *clanfile;
     t_clan *clan = (t_clan *) data;
-    if (!(clanfile = malloc(strlen(clansdir) + 1 + 4 + 1)))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for filename");
-	return -1;
-    }
 
+    clanfile = xmalloc(strlen(clansdir) + 1 + 4 + 1);
     sprintf(clanfile, "%s/%c%c%c%c", clansdir, clan->clantag >> 24, (clan->clantag >> 16) & 0xff, (clan->clantag >> 8) & 0xff, clan->clantag & 0xff);
 
     if ((fp = fopen(clanfile, "w")) == NULL)
     {
 	eventlog(eventlog_level_error, __FUNCTION__, "can't open clanfile \"%s\"", clanfile);
-	free((void *) clanfile);
+	xfree((void *) clanfile);
 	return -1;
     }
 
@@ -866,26 +688,23 @@ static int file_write_clan(void *data)
     }
 
     fclose(fp);
-    free((void *) clanfile);
+    xfree((void *) clanfile);
     return 0;
 }
 
 static int file_remove_clan(int clantag)
 {
     char *tempname;
-    if (!(tempname = malloc(strlen(clansdir) + 1 + 4 + 1)))
-    {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not allocate memory for pathname");
-	return -1;
-    }
+
+    tempname = xmalloc(strlen(clansdir) + 1 + 4 + 1);
     sprintf(tempname, "%s/%c%c%c%c", clansdir, clantag >> 24, (clantag >> 16) & 0xff, (clantag >> 8) & 0xff, clantag & 0xff);
     if (remove((const char *) tempname) < 0)
     {
-	eventlog(eventlog_level_error, __FUNCTION__, "could not delete clan file \"%s\" (remove: %s)", (char *) tempname, strerror(errno));
-	free(tempname);
+	eventlog(eventlog_level_error, __FUNCTION__, "could not delete clan file \"%s\" (remove: %s)", (char *) tempname, pstrerror(errno));
+	xfree(tempname);
 	return -1;
     }
-    free(tempname);
+    xfree(tempname);
     return 0;
 }
 
@@ -893,3 +712,205 @@ static int file_remove_clanmember(int uid)
 {
     return 0;
 }
+
+static int file_load_teams(t_load_teams_func cb)
+{
+    char const *dentry;
+    t_pdir *teamdir;
+    char *pathname;
+    unsigned int teamid;
+    t_team *team;
+    FILE *fp;
+    char * line;
+    unsigned int fteamid,lastgame;
+    unsigned char size;
+    char clienttag[5];
+    int i;
+
+    if (cb == NULL)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "get NULL callback");
+	return -1;
+    }
+
+    if (!(teamdir = p_opendir(teamsdir)))
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to open team directory \"%s\" for reading (p_opendir: %s)", teamsdir, pstrerror(errno));
+	return -1;
+    }
+    eventlog(eventlog_level_trace, __FUNCTION__, "start reading teams");
+
+    pathname = xmalloc(strlen(teamsdir) + 1 + 8 + 1);
+    while ((dentry = p_readdir(teamdir)) != NULL)
+    {
+	if (dentry[0] == '.')
+	    continue;
+
+	if (strlen(dentry) != 8)
+	{
+	    eventlog(eventlog_level_error, __FUNCTION__, "found invalid team filename in teamdir \"%s\"", dentry);
+	    continue;
+	}
+
+	sprintf(pathname, "%s/%s", teamsdir, dentry);
+
+	teamid = (unsigned int)strtoul(dentry,NULL,16); // we use hexadecimal teamid as filename
+
+	if ((fp = fopen(pathname, "r")) == NULL)
+	{
+	    eventlog(eventlog_level_error, __FUNCTION__, "can't open teamfile \"%s\"", pathname);
+	    continue;
+	}
+
+	team = xmalloc(sizeof(t_team));
+	team->teamid = teamid;
+
+	if (!(line = file_get_line(fp)))
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: file is empty");
+	    goto load_team_failure;
+	}
+	
+	if (sscanf(line,"%u,%c,%4s,%u",&fteamid,&size,clienttag,&lastgame)!=4)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: invalid number of arguments on first line");
+	    goto load_team_failure;
+	}
+	
+	if (fteamid != teamid)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: filename and stored teamid don't match");
+	    goto load_team_failure;
+	}
+	
+	size -='0';
+	if ((size<2) || (size>4))
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: invalid team size");
+	    goto load_team_failure;
+	}
+	team->size = size;
+	
+	if (!(tag_check_client(team->clienttag = tag_str_to_uint(clienttag))))
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: invalid clienttag");
+	    goto load_team_failure;
+	}
+	team->lastgame = (time_t)lastgame;
+
+	if (!(line= file_get_line(fp)))
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: missing 2nd line");
+	    goto load_team_failure;
+	}
+
+	if (sscanf(line,"%u,%u,%u,%u",&team->teammembers[0],&team->teammembers[1],&team->teammembers[2],&team->teammembers[3])!=4)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: invalid number of arguments on 2nd line");
+	    goto load_team_failure;
+	}
+
+	for (i=0; i<MAX_TEAMSIZE;i++)
+	{
+	   if (i<size)
+	   {
+		if ((team->teammembers[i]==0))
+		{
+	    		eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: too few members");
+	    		goto load_team_failure;
+		}
+	   }
+	   else
+	   {
+	   	if ((team->teammembers[i]!=0))
+		{
+	    		eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: too many members");
+	    		goto load_team_failure;
+		}
+
+	   }
+	   team->members[i] = NULL;
+	}
+
+	if (!(line= file_get_line(fp)))
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: missing 3rd line");
+	    goto load_team_failure;
+	}
+
+	if (sscanf(line,"%d,%d,%d,%d,%d",&team->wins,&team->losses,&team->xp,&team->level,&team->rank)!=5)
+	{
+	    eventlog(eventlog_level_error,__FUNCTION__,"invalid team file: invalid number of arguments on 3rd line");
+	    goto load_team_failure;
+	}
+
+	eventlog(eventlog_level_trace,__FUNCTION__,"succesfully loaded team %s",dentry);
+	cb(team);
+	
+	goto load_team_success;
+	load_team_failure:
+	  xfree((void*)team);
+	  eventlog(eventlog_level_error,__FUNCTION__,"error while reading file \"%s\"",dentry);
+
+	load_team_success:
+
+	file_get_line(NULL); // clear file_get_line buffer
+	fclose(fp);
+
+
+    }
+
+    xfree((void *) pathname);
+
+    if (p_closedir(teamdir) < 0)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "unable to close team directory \"%s\" (p_closedir: %s)", teamsdir, pstrerror(errno));
+    }
+    eventlog(eventlog_level_trace, __FUNCTION__, "finished reading teams");
+
+    return 0;
+}
+
+static int file_write_team(void *data)
+{
+    FILE *fp;
+    char *teamfile;
+    t_team *team = (t_team *) data;
+
+    teamfile = xmalloc(strlen(teamsdir) + 1 + 8 + 1);
+    sprintf(teamfile, "%s/%08x", teamsdir, team->teamid);
+
+    if ((fp = fopen(teamfile, "w")) == NULL)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "can't open teamfile \"%s\"", teamfile);
+	xfree((void *) teamfile);
+	return -1;
+    }
+
+    fprintf(fp,"%u,%c,%s,%u\n",team->teamid,team->size+'0',clienttag_uint_to_str(team->clienttag),(unsigned int)team->lastgame);
+    fprintf(fp,"%u,%u,%u,%u\n",team->teammembers[0],team->teammembers[1],team->teammembers[2],team->teammembers[3]);
+    fprintf(fp,"%d,%d,%d,%d,%d\n",team->wins,team->losses,team->xp,team->level,team->rank);
+
+    fclose(fp);
+    xfree((void *) teamfile);
+    
+    return 0;
+}
+
+static int file_remove_team(unsigned int teamid)
+{
+
+    char *tempname;
+
+    tempname = xmalloc(strlen(clansdir) + 1 + 8 + 1);
+    sprintf(tempname, "%s/%08x", clansdir, teamid);
+    if (remove((const char *) tempname) < 0)
+    {
+	eventlog(eventlog_level_error, __FUNCTION__, "could not delete team file \"%s\" (remove: %s)", (char *) tempname, pstrerror(errno));
+	xfree(tempname);
+	return -1;
+    }
+    xfree(tempname);
+    
+    return 0;
+}
