@@ -42,16 +42,13 @@
 #  include <malloc.h>
 # endif
 #endif
-
-#ifdef WIN32_GUI
-# include <win32/winmain.h>
-#endif
-
 #include "common/eventlog.h"
 #include "common/packet.h"
 #include "common/tag.h"
 #include "common/list.h"
 #include "common/util.h"
+#include "common/xalloc.h"
+#include "compat/strerror.h"
 #include "account.h"
 #include "anongame_maplists.h"
 #include "tournament.h"
@@ -68,8 +65,7 @@ static t_tournament_user * tournament_get_user(t_account * account);
 /*****/
 static int tournamentlist_create(void)
 {
-    if (!(tournament_head = list_create()))
-	return -1;
+    tournament_head = list_create();
     return 0;
 }
 
@@ -86,13 +82,13 @@ static int gamelist_destroy(void)
 		continue;
 	    }
 	    
-	    if (list_remove_elem(tournament_head,curr)<0)
+	    if (list_remove_elem(tournament_head,&curr)<0)
 		eventlog(eventlog_level_error,__FUNCTION__,"could not remove item from list");
 	    
 	    if (user->name)
-		free((void *)user->name); /* avoid warning */
+		xfree((void *)user->name); /* avoid warning */
 	    
-	    free(user);
+	    xfree(user);
 	    
 	}
 	
@@ -104,11 +100,11 @@ static int gamelist_destroy(void)
 }
 
 /*****/
-extern int tournament_check_client(char const * clienttag)
+extern int tournament_check_client(t_clienttag clienttag)
 {
-    if ((strcmp(clienttag,CLIENTTAG_WAR3XP) == 0) && (tournament_info->game_client == 2))
+    if (clienttag==CLIENTTAG_WAR3XP_UINT && tournament_info->game_client==2)
 	return 1;
-    if ((strcmp(clienttag,CLIENTTAG_WARCRAFT3) == 0) && (tournament_info->game_client == 1))
+    if (clienttag==CLIENTTAG_WARCRAFT3_UINT && tournament_info->game_client==1)
 	return 1;
 	
     return -1;
@@ -126,24 +122,15 @@ extern int tournament_signup_user(t_account * account)
 	return 0;
     }
     
-    if (!(user = malloc(sizeof(t_tournament_user)))) {
-	eventlog(eventlog_level_error,__FUNCTION__,"could not allocate memory for tournament user");
-	return -1;
-    }
-    
-    user->name		= strdup(account_get_name(account));
+    user = xmalloc(sizeof(t_tournament_user));
+    user->name		= xstrdup(account_get_name(account));
     user->wins		= 0;
     user->losses	= 0;
     user->ties		= 0;
     user->in_game	= 0;
     user->in_finals	= 0;
         
-    if (list_prepend_data(tournament_head,user)<0) {
-	eventlog(eventlog_level_error,__FUNCTION__,"could not insert user");
-	free((void *)user->name); /* avoid warning */
-	free(user);
-	return -1;
-    }
+    list_prepend_data(tournament_head,user);
     
     eventlog(eventlog_level_info,__FUNCTION__,"added user \"%s\" to tournament",account_get_name(account));
     return 0;
@@ -326,11 +313,11 @@ extern int tournament_init(char const * filename)
     char *sponsor = NULL;
     char *have_sponsor = NULL;
     char *have_icon = NULL;
-    struct tm * timestamp = malloc(sizeof(struct tm));
+    struct tm * timestamp = xmalloc(sizeof(struct tm));
     
     sprintf(format,"%%02u/%%02u/%%04u %%02u:%%02u:%%02u");
 
-    tournament_info = malloc(sizeof(t_tournament_info));
+    tournament_info = xmalloc(sizeof(t_tournament_info));
     tournament_info->start_preliminary	= 0;
     tournament_info->end_signup		= 0;
     tournament_info->end_preliminary	= 0;
@@ -343,28 +330,27 @@ extern int tournament_init(char const * filename)
     tournament_info->game_type		= 1; /* Default to 1v1 */
     tournament_info->game_client	= 2; /* Default to FT */
     tournament_info->races		= 0x3F; /* Default to all races */
-    tournament_info->format		= strdup("");
-    tournament_info->sponsor		= strdup("");
+    tournament_info->format		= xstrdup("");
+    tournament_info->sponsor		= xstrdup("");
     tournament_info->thumbs_down	= 0;
     
     anongame_tournament_maplists_destroy();
     
     if (!filename) {
 	eventlog(eventlog_level_error,__FUNCTION__,"got NULL filename");
-        free((void *)timestamp);
+        xfree((void *)timestamp);
 	return -1;
     }
     
     if (!(fp = fopen(filename,"r"))) {
-	eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for reading (fopen: %s)",filename,strerror(errno));
-	free((void *)timestamp);
+	eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for reading (fopen: %s)",filename,pstrerror(errno));
+	xfree((void *)timestamp);
 	return -1;
     }
     
     for (line=1; (buff = file_get_line(fp)); line++) {
 	for (pos=0; buff[pos]=='\t' || buff[pos]==' '; pos++);
 	if (buff[pos]=='\0' || buff[pos]=='#') {
-	    free(buff);
 	    continue;
 	}
 	if ((temp = strrchr(buff,'#'))) {
@@ -378,13 +364,12 @@ extern int tournament_init(char const * filename)
 	}
 	
 	if (strcmp(buff,"[MAPS]") == 0) {
-	    char *clienttag, *ctag, *mapname, *mname;
+	    char *clienttag, *mapname, *mname;
+	    t_clienttag ctag;
 	    
-	    free(buff);
 	    for (; (buff = file_get_line(fp));) {
 		for (pos=0; buff[pos]=='\t' || buff[pos]==' '; pos++);
 		if (buff[pos]=='\0' || buff[pos]=='#') {
-		    free(buff);
 		    continue;
 		}
 		if ((temp = strrchr(buff,'#'))) {
@@ -396,27 +381,27 @@ extern int tournament_init(char const * filename)
 		    for (endpos=len-1; buff[endpos]=='\t' || buff[endpos]==' '; endpos--);
 		    buff[endpos+1] = '\0';
 		}
-		
+		/* FIXME: use next_token() */
 		if (!(clienttag = strtok(buff, " \t"))) { /* strtok modifies the string it is passed */
-		    free(buff);
+		    continue;
+		}
+		if (strlen(clienttag) != 4) {
 		    continue;
 		}
 		if (strcmp(buff,"[ENDMAPS]") == 0) {
-		    free(buff);
 		    break;
 		}
 		if (!(mapname = strtok(NULL," \t"))) {
-		    free(buff);
 		    continue;
 		}
-		ctag = strdup(clienttag);
-		mname = strdup(mapname);
+		if (!tag_check_client((ctag = tag_case_str_to_uint(clienttag)))) {
+		    continue;
+		}
+		mname = xstrdup(mapname);
 		
 		anongame_add_tournament_map(ctag, mname);
-		eventlog(eventlog_level_trace,__FUNCTION__,"added tournament map \"%s\" for %s",mname,ctag);
-		free(ctag);
-		free(mname);
-		free(buff);
+		eventlog(eventlog_level_trace,__FUNCTION__,"added tournament map \"%s\" for %s",mname,clienttag);
+		xfree(mname);
 	    }
 	} else {
 	    variable = buff;
@@ -615,8 +600,8 @@ extern int tournament_init(char const * filename)
 	        pointer = strchr(pointer,'\"');
 	        pointer[0]='\0';
 	        
-	        if (tournament_info->format) free((void *)tournament_info->format);
-	        tournament_info->format = strdup(value);
+	        if (tournament_info->format) xfree((void *)tournament_info->format);
+	        tournament_info->format = xstrdup(value);
 	    }
 	    else if (strcmp(variable,"races") == 0) {
 	        unsigned int intvalue = 0;
@@ -648,7 +633,7 @@ extern int tournament_init(char const * filename)
 	        pointer = strchr(pointer,'\"');
 	        pointer[0]='\0';
 		
-	        have_sponsor = strdup(value);
+	        have_sponsor = xstrdup(value);
 	    }
 	    else if (strcmp(variable,"icon") == 0) {
 	        pointer = strchr(pointer,'\"');
@@ -657,7 +642,7 @@ extern int tournament_init(char const * filename)
 	        pointer = strchr(pointer,'\"');
 	        pointer[0]='\0';
 		
-	        have_icon = strdup(value);
+	        have_icon = xstrdup(value);
 	    }
 	    else if (strcmp(variable,"thumbs_down") == 0) {
 	        tournament_info->thumbs_down = atoi(pointer);
@@ -666,7 +651,7 @@ extern int tournament_init(char const * filename)
 	        eventlog(eventlog_level_error,__FUNCTION__,"bad option \"%s\" in \"%s\"",variable,filename);
 	    
 	    if (have_sponsor && have_icon) {
-	        sponsor = malloc(strlen(have_sponsor)+6);
+	        sponsor = xmalloc(strlen(have_sponsor)+6);
 		
 		if (strlen(have_icon) == 4)
 		    sprintf(sponsor, "%c%c%c%c,%s",have_icon[3],have_icon[2],have_icon[1],have_icon[0],have_sponsor);
@@ -678,21 +663,21 @@ extern int tournament_init(char const * filename)
 		}
 		
 		if (tournament_info->sponsor)
-		    free((void *)tournament_info->sponsor);
+		    xfree((void *)tournament_info->sponsor);
 	        
-		tournament_info->sponsor = strdup(sponsor);
-	        free((void *)have_sponsor);
-		free((void *)have_icon);
-	        free((void *)sponsor);
+		tournament_info->sponsor = xstrdup(sponsor);
+	        xfree((void *)have_sponsor);
+		xfree((void *)have_icon);
+	        xfree((void *)sponsor);
 		have_sponsor = NULL;
 	        have_icon = NULL;
 	    }
-	    free(buff);
 	}
     }
-    if (have_sponsor) free((void *)have_sponsor);
-    if (have_icon) free((void *)have_icon);
-    free((void *)timestamp);
+    if (have_sponsor) xfree((void *)have_sponsor);
+    if (have_icon) xfree((void *)have_icon);
+    xfree((void *)timestamp);
+    file_get_line(NULL); // clear file_get_line buffer
     fclose(fp);
     
     /* check if we have timestamps for all the times */ 
@@ -704,10 +689,7 @@ extern int tournament_init(char const * filename)
 	tournament_info->start_preliminary = 0;
 	eventlog(eventlog_level_warn,__FUNCTION__,"one or more timestamps for tournaments is not valid, tournament has been disabled");
     } else {
-    	if (tournamentlist_create()<0) {
-	    tournament_info->start_preliminary = 0;
-	    eventlog(eventlog_level_warn,__FUNCTION__,"unable to create tournament list, tournament has been disabled");
-	}
+    	tournamentlist_create();
     }
     
     return 0;
@@ -715,9 +697,9 @@ extern int tournament_init(char const * filename)
 
 extern int tournament_destroy(void)
 {
-    if (tournament_info->format) free((void *)tournament_info->format);
-    if (tournament_info->sponsor) free((void *)tournament_info->sponsor);
-    if (tournament_info) free((void *)tournament_info);
+    if (tournament_info->format) xfree((void *)tournament_info->format);
+    if (tournament_info->sponsor) xfree((void *)tournament_info->sponsor);
+    if (tournament_info) xfree((void *)tournament_info);
     tournament_info = NULL;
     gamelist_destroy();
     return 0;
