@@ -87,6 +87,7 @@
 #include "common/util.h"
 #include "common/tracker.h"
 #include "common/xalloc.h"
+#include "common/bn_type.h"
 #include "common/setup_after.h"
 
 
@@ -121,7 +122,7 @@ typedef struct
 static int server_process(int sockfd);
 static void usage(char const * progname);
 static void getprefs(int argc, char * argv[]);
-static void fixup_str(char * str);
+static void fixup_str(bn_byte * str, unsigned int size);
 
 
 /******************************************************************************
@@ -319,41 +320,41 @@ static int server_process(int sockfd)
 		   if (prefs.XML_mode == 1)
 		  {
 		    fprintf(outfile,"<server>\n\t<address>%s</address>\n",inet_ntoa(server->address));
-		    fprintf(outfile,"\t<port>%hu</port>\n",(unsigned short)ntohs(server->info.port));
+		    fprintf(outfile,"\t<port>%hu</port>\n",bn_short_nget(server->info.port));
 		    fprintf(outfile,"\t<location>%s</location>\n",server->info.server_location);
 		    fprintf(outfile,"\t<software>%s</software>\n",server->info.software);
 		    fprintf(outfile,"\t<version>%s</version>\n",server->info.version);
-		    fprintf(outfile,"\t<users>%lu</users>\n",(unsigned long)ntohl(server->info.users));
-		    fprintf(outfile,"\t<channels>%lu</channels>\n",(unsigned long)ntohl(server->info.channels));
-		    fprintf(outfile,"\t<games>%lu</games>\n",(unsigned long)ntohl(server->info.games));
+		    fprintf(outfile,"\t<users>%lu</users>\n",bn_int_nget(server->info.users));
+		    fprintf(outfile,"\t<channels>%lu</channels>\n",bn_int_nget(server->info.channels));
+		    fprintf(outfile,"\t<games>%lu</games>\n",bn_int_nget(server->info.games));
 		    fprintf(outfile,"\t<description>%s</description>\n",server->info.server_desc);
 		    fprintf(outfile,"\t<platform>%s</platform>\n",server->info.platform);
 		    fprintf(outfile,"\t<url>%s</url>\n",server->info.server_url);
 		    fprintf(outfile,"\t<contact_name>%s</contact_name>\n",server->info.contact_name);
 		    fprintf(outfile,"\t<contact_email>%s</contact_email>\n",server->info.contact_email);
-		    fprintf(outfile,"\t<uptime>%lu</uptime>\n",(unsigned long)ntohl(server->info.uptime));
-		    fprintf(outfile,"\t<total_games>%lu</total_games>\n",(unsigned long)ntohl(server->info.total_games));
-		    fprintf(outfile,"\t<logins>%lu</logins>\n",(unsigned long)ntohl(server->info.total_logins));
+		    fprintf(outfile,"\t<uptime>%lu</uptime>\n",bn_int_nget(server->info.uptime));
+		    fprintf(outfile,"\t<total_games>%lu</total_games>\n",bn_int_nget(server->info.total_games));
+		    fprintf(outfile,"\t<logins>%lu</logins>\n",bn_int_nget(server->info.total_logins));
 		    fprintf(outfile,"</server>\n");
 		  }
 		  else
 		  { 
 		    fprintf(outfile,"%s\n##\n",inet_ntoa(server->address));
-		    fprintf(outfile,"%hu\n##\n",(unsigned short)ntohs(server->info.port));
+		    fprintf(outfile,"%hu\n##\n",bn_short_nget(server->info.port));
 		    fprintf(outfile,"%s\n##\n",server->info.server_location);
 		    fprintf(outfile,"%s\n##\n",server->info.software);
 		    fprintf(outfile,"%s\n##\n",server->info.version);
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.users));
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.channels));
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.games));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.users));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.channels));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.games));
 		    fprintf(outfile,"%s\n##\n",server->info.server_desc);
 		    fprintf(outfile,"%s\n##\n",server->info.platform);
 		    fprintf(outfile,"%s\n##\n",server->info.server_url);
 		    fprintf(outfile,"%s\n##\n",server->info.contact_name);
 		    fprintf(outfile,"%s\n##\n",server->info.contact_email);
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.uptime));
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.total_games));
-		    fprintf(outfile,"%lu\n##\n",(unsigned long)ntohl(server->info.total_logins));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.uptime));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.total_games));
+		    fprintf(outfile,"%lu\n##\n",bn_int_nget(server->info.total_logins));
 		    fprintf(outfile,"###\n");
 		  }
 		}
@@ -391,32 +392,24 @@ static int server_process(int sockfd)
 	    if (psock_recvfrom(sockfd,&packet,sizeof(packet),0,(struct sockaddr *)&cliaddr,&len)>=0)
 	    {
 		
-		if (ntohs(packet.packet_version)>=TRACK_VERSION)
+		if (bn_short_nget(packet.packet_version)>=TRACK_VERSION)
 		{
-		    packet.software[sizeof(packet.software)-1] = '\0';
-		    if (strstr(packet.software,"##"))
-			fixup_str(packet.software);
-		    packet.version[sizeof(packet.version)-1] = '\0';
-		    if (strstr(packet.version,"##"))
-			fixup_str(packet.version);
-		    packet.platform[sizeof(packet.platform)-1] = '\0';
-		    if (strstr(packet.platform,"##"))
-			fixup_str(packet.platform);
-		    packet.server_desc[sizeof(packet.server_desc)-1] = '\0';
-		    if (strstr(packet.server_desc,"##"))
-			fixup_str(packet.server_desc);
-		    packet.server_location[sizeof(packet.server_location)-1] = '\0';
-		    if (strstr(packet.server_location,"##"))
-			fixup_str(packet.server_location);
-		    packet.server_url[sizeof(packet.server_url)-1] = '\0';
-		    if (strstr(packet.server_url,"##"))
-			fixup_str(packet.server_url);
-		    packet.contact_name[sizeof(packet.contact_name)-1] = '\0';
-		    if (strstr(packet.contact_name,"##"))
-			fixup_str(packet.contact_name);
-		    packet.contact_email[sizeof(packet.contact_email)-1] = '\0';
-		    if (strstr(packet.contact_email,"##"))
-			fixup_str(packet.contact_email);
+		    bn_byte_set(&packet.software[sizeof(packet.software)-1],'\0');
+		    fixup_str(packet.software,sizeof(packet.software));
+		    bn_byte_set(&packet.version[sizeof(packet.version)-1],'\0');
+		    fixup_str(packet.version,sizeof(packet.version));
+		    bn_byte_set(&packet.platform[sizeof(packet.platform)-1],'\0');
+		    fixup_str(packet.platform,sizeof(packet.platform));
+		    bn_byte_set(&packet.server_desc[sizeof(packet.server_desc)-1],'\0');
+		    fixup_str(packet.server_desc,sizeof(packet.server_desc));
+		    bn_byte_set(&packet.server_location[sizeof(packet.server_location)-1],'\0');
+		    fixup_str(packet.server_location,sizeof(packet.server_location));
+		    bn_byte_set(&packet.server_url[sizeof(packet.server_url)-1],'\0');
+		    fixup_str(packet.server_url,sizeof(packet.server_url));
+		    bn_byte_set(&packet.contact_name[sizeof(packet.contact_name)-1],'\0');
+		    fixup_str(packet.contact_name,sizeof(packet.contact_name));
+		    bn_byte_set(&packet.contact_email[sizeof(packet.contact_email)-1],'\0');
+		    fixup_str(packet.contact_email,sizeof(packet.contact_email));
 		    
 		    /* Find this server's slot */
 		    LIST_TRAVERSE(serverlist_head,curr)
@@ -425,7 +418,7 @@ static int server_process(int sockfd)
 			
 			if (!memcmp(&server->address,&cliaddr.sin_addr,sizeof(struct in_addr)))
 			{
-			    if (ntohl(packet.flags)&TF_SHUTDOWN)
+			    if (bn_int_nget(packet.flags)&TF_SHUTDOWN)
 			    {
 				list_remove_elem(serverlist_head,&curr);
 				xfree(server);
@@ -441,7 +434,7 @@ static int server_process(int sockfd)
 		    }
 		    
 		    /* Not found? Make a new slot */
-		    if (!(ntohl(packet.flags)&TF_SHUTDOWN) && !curr)
+		    if (!(bn_int_nget(packet.flags)&TF_SHUTDOWN) && !curr)
 		    {
 			server = xmalloc(sizeof(t_server));
 			server->address = cliaddr.sin_addr;
@@ -468,9 +461,9 @@ static int server_process(int sockfd)
 			     " total_games=%lu"
 			     " total_logins=%lu",
 			     inet_ntoa(cliaddr.sin_addr),
-			     ntohs(packet.packet_version),
-			     (unsigned long)ntohl(packet.flags),
-			     ntohs(packet.port),
+			     bn_short_nget(packet.packet_version),
+			     bn_int_nget(packet.flags),
+			     bn_short_nget(packet.port),
 			     packet.software,
 			     packet.version,
 			     packet.platform,
@@ -479,9 +472,9 @@ static int server_process(int sockfd)
 			     packet.server_url,
 			     packet.contact_name,
 			     packet.contact_email,
-			     (unsigned long)ntohl(packet.uptime),
-			     (unsigned long)ntohl(packet.total_games),
-			     (unsigned long)ntohl(packet.total_logins));
+			     bn_int_nget(packet.uptime),
+			     bn_int_nget(packet.total_games),
+			     bn_int_nget(packet.total_logins));
 		}
 		
 	    }
@@ -774,12 +767,15 @@ static void getprefs(int argc, char * argv[])
 }
 
 
-static void fixup_str(char * str)
+void fixup_str(bn_byte * str, unsigned int size)
 {
-    char         prev;
+    bn_byte         prev;
     unsigned int i;
-    
-    for (prev='\0',i=0; i<strlen(str); prev=str[i],i++)
-	if (prev=='#' && str[i]=='#')
-	    str[i] = '%';
+
+    bn_byte_set(&prev,'\0');
+
+    for (i=0; i<size, bn_byte_get(str[i])!='\0'; bn_byte_set(&prev,bn_byte_get(str[i])),i++)
+	if (bn_byte_get(prev)=='#' && bn_byte_get(str[i])=='#')
+	    bn_byte_set(&str[i],'%');
 }
+
