@@ -102,6 +102,7 @@
 #endif
 #include "common/hexdump.h"
 #include "client.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 
@@ -179,6 +180,7 @@ extern int main(int argc, char * argv[])
     char               timestr[FILE_TIME_MAXLEN];
     unsigned int       screen_width,screen_height;
     int                munged;
+    unsigned int       newproto = 0;
 
     if (argc<1 || !argv || !argv[0])
     {
@@ -277,6 +279,7 @@ extern int main(int argc, char * argv[])
                 usage(argv[0]);
             }
             clienttag = CLIENTTAG_WARCRAFT3;
+	    newproto = 1;
         }
         else if (strcmp(argv[a],"--client=W3XP")==0)
         {
@@ -286,6 +289,7 @@ extern int main(int argc, char * argv[])
                 usage(argv[0]);
             }
             clienttag = CLIENTTAG_WAR3XP;
+	    newproto = 1;
         }
         else if (strncmp(argv[a],"--client=",9)==0)
         {
@@ -408,7 +412,7 @@ extern int main(int argc, char * argv[])
     if (hexfile)
     {
 	if (!(hexstrm = fopen(hexfile,"w")))
-	    fprintf(stderr,"%s: could not open file \"%s\" for writing the hexdump (fopen: %s)",argv[0],hexfile,strerror(errno));
+	    fprintf(stderr,"%s: could not open file \"%s\" for writing the hexdump (fopen: %s)",argv[0],hexfile,pstrerror(errno));
 	else
 	  { fprintf(hexstrm,"# dump generated by bnftp version "PVPGN_VERSION"\n"); }
     }
@@ -451,7 +455,7 @@ extern int main(int argc, char * argv[])
     
     if ((sd = psock_socket(PSOCK_PF_INET,PSOCK_SOCK_STREAM,PSOCK_IPPROTO_TCP))<0)
     {
-	fprintf(stderr,"%s: could not create socket (psock_socket: %s)\n",argv[0],strerror(psock_errno()));
+	fprintf(stderr,"%s: could not create socket (psock_socket: %s)\n",argv[0],pstrerror(psock_errno()));
 	if (changed_in)
 	    tcsetattr(fd_stdin,TCSAFLUSH,&in_attr_old);
 	return STATUS_FAILURE;
@@ -463,7 +467,7 @@ extern int main(int argc, char * argv[])
     memcpy(&saddr.sin_addr.s_addr,host->h_addr_list[0],host->h_length);
     if (psock_connect(sd,(struct sockaddr *)&saddr,sizeof(saddr))<0)
     {
-	fprintf(stderr,"%s: could not connect to server \"%s\" port %hu (psock_connect: %s)\n",argv[0],servname,servport,strerror(psock_errno()));
+	fprintf(stderr,"%s: could not connect to server \"%s\" port %hu (psock_connect: %s)\n",argv[0],servname,servport,pstrerror(psock_errno()));
 	if (changed_in)
 	    tcsetattr(fd_stdin,TCSAFLUSH,&in_attr_old);
 	return STATUS_FAILURE;
@@ -611,16 +615,7 @@ extern int main(int argc, char * argv[])
 		unsigned int bnr;
 		int          renamed=0;
 		
-		if (!(bakfile = malloc(strlen(reqfile)+1+2+1))) /* assuming we go up to bnr 99 we need reqfile+'.'+'99'+'\0' */
-		{
-		    fprintf(stderr,"%s: unable to allocate memory for backup filename.\n",argv[0]);
-		    if (changed_in)
-			tcsetattr(fd_stdin,TCSAFLUSH,&in_attr_old);
-		    packet_del_ref(fpacket);
-		    packet_del_ref(rpacket);
-		    return STATUS_FAILURE;
-		}
-		
+		bakfile = xmalloc(strlen(reqfile)+1+2+1); /* assuming we go up to bnr 99 we need reqfile+'.'+'99'+'\0' */
 		for (bnr=0; bnr<100; bnr++)
 		{
 		    sprintf(bakfile,"%s.%d",reqfile,bnr);
@@ -628,7 +623,7 @@ extern int main(int argc, char * argv[])
 			continue; /* backup exists */
 		    /* backup does not exist */
 		    if (rename(reqfile,bakfile)<0) /* just rename the existing file to the backup */
-			fprintf(stderr,"%s: could not create backup file \"%s\" (rename: %s)\n",argv[0],bakfile,strerror(errno));
+			fprintf(stderr,"%s: could not create backup file \"%s\" (rename: %s)\n",argv[0],bakfile,pstrerror(errno));
 		    else
 		    {
 			renamed = 1;
@@ -636,7 +631,7 @@ extern int main(int argc, char * argv[])
 		    }
 		    break;
 		}
-		free(bakfile);
+		xfree(bakfile);
 		if (!renamed)
 		{
 		    fprintf(stderr,"%s: could not create backup for \"%s\".\n",argv[0],reqfile);
@@ -670,15 +665,24 @@ extern int main(int argc, char * argv[])
 	packet_del_ref(rpacket);
 	return STATUS_FAILURE;
     }
-    packet_set_size(packet,sizeof(t_client_file_req));
-    packet_set_type(packet,CLIENT_FILE_REQ);
-    bn_int_tag_set(&packet->u.client_file_req.archtag,archtag);
-    bn_int_tag_set(&packet->u.client_file_req.clienttag,clienttag);
-    bn_int_set(&packet->u.client_file_req.adid,0);
-    bn_int_set(&packet->u.client_file_req.extensiontag,0);
-    bn_int_set(&packet->u.client_file_req.startoffset,startoffset);
-    bn_long_set_a_b(&packet->u.client_file_req.timestamp,0x00000000,0x00000000);
-    packet_append_string(packet,reqfile);
+    if (newproto) {
+	/* first send ARCH/CTAG */
+	packet_set_size(packet, sizeof(t_client_file_req2));
+	packet_set_type(packet, CLIENT_FILE_REQ2);
+	bn_int_tag_set(&packet->u.client_file_req2.archtag,archtag);
+	bn_int_tag_set(&packet->u.client_file_req2.clienttag,clienttag);
+	bn_long_set_a_b(&packet->u.client_file_req2.unknown1,0,0);
+    } else {
+	packet_set_size(packet,sizeof(t_client_file_req));
+	packet_set_type(packet, CLIENT_FILE_REQ);
+	bn_int_tag_set(&packet->u.client_file_req.archtag,archtag);
+	bn_int_tag_set(&packet->u.client_file_req.clienttag,clienttag);
+	bn_int_set(&packet->u.client_file_req.adid,0);
+	bn_int_set(&packet->u.client_file_req.extensiontag,0);
+	bn_int_set(&packet->u.client_file_req.startoffset,startoffset);
+	bn_long_set_a_b(&packet->u.client_file_req.timestamp,0x00000000,0x00000000);
+	packet_append_string(packet,reqfile);
+    }
     if (hexstrm)
     {
 	fprintf(hexstrm,"%d: send class=%s[0x%02hx] type=%s[0x%04hx] length=%u\n",
@@ -688,11 +692,55 @@ extern int main(int argc, char * argv[])
 		packet_get_size(packet));
 	hexdump(hexstrm,packet_get_raw_data(packet,0),packet_get_size(packet));
     }
-    printf("\nRequesting info...");
+    if (newproto) printf("\nSending ARCH/CTAG info...");
+    else printf("\nRequesting info...");
     fflush(stdout);
     client_blocksend_packet(sd,packet);
     packet_del_ref(packet);
     
+    if (newproto) {
+	/* received the stupid 4 bytes unknown "packet" */
+	packet_set_size(fpacket, sizeof(t_server_file_unknown1));
+	if (client_blockrecv_packet(sd,fpacket)<0)
+	{
+	    fprintf(stderr,"%s: server closed connection\n",argv[0]);
+	    packet_del_ref(fpacket);
+	    packet_del_ref(rpacket);
+	    return STATUS_FAILURE;
+	}
+	if (hexstrm)
+	{
+	    fprintf(hexstrm,"%d: recv class=%s[0x%02hx] type=%s[0x%04hx] length=%u\n",
+		     sd,
+		     packet_get_class_str(fpacket),(unsigned int)packet_get_class(fpacket),
+		     packet_get_type_str(fpacket,packet_dir_from_server),packet_get_type(fpacket),
+		     packet_get_size(fpacket));
+	    hexdump(hexstrm,packet_get_raw_data(fpacket,0),packet_get_size(fpacket));
+	}
+
+	/* send file info request */
+	packet_set_size(fpacket, sizeof(t_client_file_req3));
+	bn_int_set(&fpacket->u.client_file_req3.unknown1, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.timestamp, 0, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.unknown2, 0, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.unknown3, 0, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.unknown4, 0, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.unknown5, 0, 0);
+	bn_long_set_a_b(&fpacket->u.client_file_req3.unknown6, 0, 0);
+	packet_append_string(fpacket, reqfile);
+	printf("\nRequesting info...");
+	fflush(stdout);
+	client_blocksend_packet(sd,fpacket);
+	if (hexstrm)
+	{
+	    fprintf(hexstrm,"%d: send class=%s[0x%02hx] type=%s[0x%04hx] length=%u\n",
+		    sd,
+		    packet_get_class_str(fpacket),(unsigned int)packet_get_class(fpacket),
+		    packet_get_type_str(fpacket,packet_dir_from_client),packet_get_type(fpacket),
+		    packet_get_size(fpacket));
+	    hexdump(hexstrm,packet_get_raw_data(fpacket,0),packet_get_size(fpacket));
+	}
+    }
     do
     {
 	if (client_blockrecv_packet(sd,rpacket)<0)
@@ -724,7 +772,7 @@ extern int main(int argc, char * argv[])
     {
 	if (!(fp = fopen(reqfile,"ab")))
 	{
-	    fprintf(stderr,"%s: could not open file \"%s\" for appending (fopen: %s)\n",argv[0],reqfile,strerror(errno));
+	    fprintf(stderr,"%s: could not open file \"%s\" for appending (fopen: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -734,7 +782,7 @@ extern int main(int argc, char * argv[])
     {
 	if (!(fp = fopen(reqfile,"wb")))
 	{
-	    fprintf(stderr,"%s: could not open file \"%s\" for writing (fopen: %s)\n",argv[0],reqfile,strerror(errno));
+	    fprintf(stderr,"%s: could not open file \"%s\" for writing (fopen: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -763,7 +811,7 @@ extern int main(int argc, char * argv[])
 	    printf("error\n");
 	    fprintf(stderr,"%s: server closed connection\n",argv[0]);
 	    if (fclose(fp)<0)
-		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,strerror(errno));
+		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -780,9 +828,9 @@ extern int main(int argc, char * argv[])
 	if (fwrite(packet_get_raw_data_const(fpacket,0),1,MAX_PACKET_SIZE,fp)<MAX_PACKET_SIZE)
 	{
 	    printf("error\n");
-	    fprintf(stderr,"%s: could not write to file \"%s\" (fwrite: %s)\n",argv[0],reqfile,strerror(errno));
+	    fprintf(stderr,"%s: could not write to file \"%s\" (fwrite: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    if (fclose(fp)<0)
-		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,strerror(errno));
+		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -799,7 +847,7 @@ extern int main(int argc, char * argv[])
 	    printf("error\n");
 	    fprintf(stderr,"%s: server closed connection\n",argv[0]);
 	    if (fclose(fp)<0)
-		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,strerror(errno));
+		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -818,7 +866,7 @@ extern int main(int argc, char * argv[])
 	    printf("error\n");
 	    fprintf(stderr,"%s: could not write to file \"%s\"\n",argv[0],reqfile);
 	    if (fclose(fp)<0)
-		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,strerror(errno));
+		fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,pstrerror(errno));
 	    packet_del_ref(fpacket);
 	    packet_del_ref(rpacket);
 	    return STATUS_FAILURE;
@@ -832,12 +880,12 @@ extern int main(int argc, char * argv[])
     {
 	fprintf(hexstrm,"# end of dump\n");
 	if (fclose(hexstrm)<0)
-	    fprintf(stderr,"%s: could not close hexdump file \"%s\" after writing (fclose: %s)",argv[0],hexfile,strerror(errno));
+	    fprintf(stderr,"%s: could not close hexdump file \"%s\" after writing (fclose: %s)",argv[0],hexfile,pstrerror(errno));
     }
     
     if (fclose(fp)<0)
     {
-	fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,strerror(errno));
+	fprintf(stderr,"%s: could not close file \"%s\" after writing (fclose: %s)\n",argv[0],reqfile,pstrerror(errno));
 	return STATUS_FAILURE;
     }
     
