@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2001  Marco Ziech (mmz@gmx.net)
  * Copyright (C) 2005  Bryan Biedenkapp (gatekeep@gmail.com)
+ * Copyright (C) 2008  Pelish (pelish@gmail.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -338,6 +339,7 @@
 #define RPL_BATTLECLAN	     358
 
 #define RPL_BAD_LOGIN        378
+#define RPL_VERCHK_NONREQ    379  /* for WOL */
 
 #define RPL_FIND_USER_EX     398
 
@@ -349,6 +351,8 @@
 #define RPL_LADDERSERV	     609
 #define RPL_LOBCOUNT	     610
 
+#define ERR_IDNOEXIST        439
+
 #define ERR_GAMECHANCLOSED   478
 
 /** error's */
