@@ -1,5 +1,4 @@
 /*
-
  * Copyright (C) 1998  Mark Baysinger (mbaysng@ucsd.edu)
  * Copyright (C) 1998,1999,2000  Ross Combs (rocombs@cs.nmsu.edu)
  *
@@ -32,8 +31,6 @@
 # include "d2cs/d2cs_protocol.h"
 # include "d2cs/d2cs_d2gs_protocol.h"
 # include "d2cs/d2cs_bnetd_protocol.h"
-# include "common/auth_protocol.h"
-# include "common/w3xp_protocol.h"
 #else
 # define JUST_NEED_TYPES
 # include "common/field_sizes.h"
@@ -47,8 +44,6 @@
 # include "d2cs/d2cs_protocol.h"
 # include "d2cs/d2cs_d2gs_protocol.h"
 # include "d2cs/d2cs_bnetd_protocol.h"
-# include "common/auth_protocol.h"
-# include "common/w3xp_protocol.h"
 # undef JUST_NEED_TYPES
 #endif
 
@@ -64,7 +59,6 @@ typedef enum
     packet_class_d2gs,
     packet_class_d2cs,
     packet_class_d2cs_bnetd,
-    packet_class_auth,
     packet_class_w3route
 } t_packet_class;
 
@@ -96,7 +90,6 @@ typedef struct
         t_file_generic   file;
         t_udp_generic    udp;
         t_d2game_generic d2game;
-	t_auth_generic   auth;
 	t_w3route_generic w3route;
         
 	t_client_initconn client_initconn;
@@ -180,8 +173,8 @@ typedef struct
         t_client_realmlistreq_110   client_realmlistreq_110;
         t_server_realmlistreply     server_realmlistreply;
         t_server_realmlistreply_110 server_realmlistreply_110;
-        t_client_realmjoinreq       client_realmjoinreq;
-        t_server_realmjoinreply     server_realmjoinreply;
+        t_client_profilereq         client_profilereq;
+        t_server_profilereply       server_profilereply;
         t_client_realmjoinreq_109   client_realmjoinreq_109;
         t_server_realmjoinreply_109 server_realmjoinreply_109;
         t_client_unknown_37         client_unknown_37;
@@ -197,33 +190,15 @@ typedef struct
 	t_client_file_req          client_file_req;
 	t_server_file_reply        server_file_reply;
 
-	/* ADDED BY UNDYING SOULZZ 4/3/02 */
 	t_server_file_unknown1	   server_file_unknown1;
-	t_client_war3113_file_req  client_war3113_file_req;	
+	t_client_file_req2         client_file_req2;
+	t_client_file_req3         client_file_req3;
 
 	t_server_udptest           server_udptest;
 	t_client_udpping           client_udpping;
 	t_client_sessionaddr1      client_sessionaddr1;
 	t_client_sessionaddr2      client_sessionaddr2;
-	
-	t_client_authloginreq      client_authloginreq;
-	t_server_authloginreply    server_authloginreply;
-	t_client_createcharreq     client_createcharreq;
-	t_server_createcharreply   server_createcharreply;
-	t_client_creategamereq     client_creategamereq;
-	t_server_creategamereply   server_creategamereply;
-	t_client_joingamereq2      client_joingamereq2;
-	t_server_joingamereply2    server_joingamereply2;
-	t_server_creategame_wait   server_creategame_wait;
-	t_client_cancel_create     client_cancel_create;
-	t_client_d2gamelistreq     client_d2gamelistreq;
-	t_server_d2gamelistreply   server_d2gamelistreply;
-	t_client_gameinforeq       client_gameinforeq;
-	t_server_gameinforeply     server_gameinforeply;
-	t_client_charloginreq      client_charloginreq;
-	t_server_charloginreply    server_charloginreply;
-	t_client_deletecharreq     client_deletecharreq;
-	t_server_deletecharreply   server_deletecharreply;
+
 	t_client_motd_w3           client_motd_w3;
 	t_server_motd_w3           server_motd_w3;
 	t_client_logonproofreq     client_logonproofreq;
@@ -235,18 +210,8 @@ typedef struct
 	t_client_findanongame_at_inv   client_findanongame_at_inv;
 
 
-	// 5/12/02 EDITED BY THEUNDYING for WC3 1.31+
 	t_server_findanongame_playgame_cancel		server_findanongame_playgame_cancel;
 	t_server_anongame_found		server_anongame_found;
-	//t_server_w3profile_reply      server_w3profile_reply;
-    t_client_ladderreq2        client_ladderreq2;
-	t_server_ladderreply2      server_ladderreply2;
-	t_client_authmotdreq       client_authmotdreq;
-	t_server_authmotdreply     server_authmotdreply;
-	t_client_charlistreq       client_charlistreq;
-	t_server_charlistreply     server_charlistreply;
-	t_client_convertcharreq    client_convertcharreq;
-	t_server_convertcharreply  server_convertcharreply;
         t_d2cs_bnetd_generic            d2cs_bnetd;
         t_bnetd_d2cs_authreq            bnetd_d2cs_authreq;
         t_d2cs_bnetd_authreply          d2cs_bnetd_authreply;
@@ -255,6 +220,8 @@ typedef struct
         t_bnetd_d2cs_accountloginreply  bnetd_d2cs_accountloginreply;
         t_d2cs_bnetd_charloginreq       d2cs_bnetd_charloginreq;
         t_bnetd_d2cs_charloginreply     bnetd_d2cs_charloginreply;
+	t_bnetd_d2cs_gameinforeq	bnetd_d2cs_gameinforeq;
+	t_d2cs_bnetd_gameinforeply	d2cs_bnetd_gameinforeply;
 
         t_d2cs_d2gs_generic             d2cs_d2gs;
         t_d2cs_d2gs_authreq             d2cs_d2gs_authreq;
@@ -272,7 +239,7 @@ typedef struct
         t_d2cs_d2gs_echoreq             d2cs_d2gs_echoreq;
         t_d2gs_d2cs_echoreply           d2gs_d2cs_echoreply;
 	t_d2cs_d2gs_control             d2cs_d2gs_control;
-	t_d2cs_d2gs_setconffile		d2cs_d2gs_setconffile;
+	t_d2cs_d2gs_setconffile			d2cs_d2gs_setconffile;
 
         t_d2cs_client_generic           d2cs_client;
         t_client_d2cs_loginreq          client_d2cs_loginreq;
@@ -304,8 +271,6 @@ typedef struct
         t_d2cs_client_charlistreply_110 d2cs_client_charlistreply_110;
         t_client_d2cs_convertcharreq    client_d2cs_convertcharreq;
         t_d2cs_client_convertcharreply  d2cs_client_convertcharreply;
-		// 5/14/02 - THEUNDYING - USED FOR THE FRIENDS LIST
-		// [zap-zero] 20020516
 
 	t_client_friendslistreq		client_friendslistreq;
 	t_server_friendslistreply	server_friendslistreply;
@@ -315,22 +280,18 @@ typedef struct
 	t_server_frienddel_ack		server_frienddel_ack;
 	t_server_friendmove_ack		server_friendmove_ack;
 
-	// 5/19/02 - THEUNDYING - used for arranged teams
-	t_client_arrangedteam_friendscreen		client_arrangedteam_friendscreen;
-	t_server_arrangedteam_friendscreen		server_arrangedteam_friendscreen;
-	t_client_arrangedteam_invite_friend		client_arrangedteam_invite_friend;
-	t_server_arrangedteam_invite_friend_ack		server_arrangedteam_invite_friend_ack;
-	t_server_arrangedteam_send_invite		server_arrangedteam_send_invite;
+	t_client_arrangedteam_friendscreen	client_arrangedteam_friendscreen;
+	t_server_arrangedteam_friendscreen	server_arrangedteam_friendscreen;
+	t_client_arrangedteam_invite_friend	client_arrangedteam_invite_friend;
+	t_server_arrangedteam_invite_friend_ack	server_arrangedteam_invite_friend_ack;
+	t_server_arrangedteam_send_invite	server_arrangedteam_send_invite;
 	t_client_arrangedteam_accept_invite    client_arrangedteam_accept_invite;
 	t_client_arrangedteam_accept_decline_invite    client_arrangedteam_accept_decline_invite;
 	t_server_arrangedteam_member_decline    server_arrangedteam_member_decline;
-	// 5/24/02 - THEUNDYING - updates for ANONGAME PROFILE PACKET
-	t_client_findanongame_profile			client_findanongame_profile;
+	t_client_findanongame_profile		client_findanongame_profile;
 
-	/* dizzy : war3 1.03 has changed format */
-	t_server_findanongame_profile2			server_findanongame_profile2;
+	t_server_findanongame_profile2		server_findanongame_profile2;
 		
-	// [zap-zero] 20020529
 	t_client_w3route_req			client_w3route_req;
 	t_server_w3route_ack			server_w3route_ack;
 	t_server_w3route_playerinfo		server_w3route_playerinfo;
@@ -348,48 +309,50 @@ typedef struct
 	t_client_findanongame_inforeq		client_findanongame_inforeq;
 	t_server_findanongame_inforeply		server_findanongame_inforeply;
 	
-	t_client_w3xp_clan_invitereq        client_w3xp_clan_invitereq;
-    t_server_w3xp_clan_invitereply      server_w3xp_clan_invitereply;
-	t_server_w3xp_clan_invitereq        server_w3xp_clan_invitereq;
-	t_client_w3xp_clan_invitereply      client_w3xp_clan_invitereply;
-	t_client_w3xp_clan_disbandreq       client_w3xp_clan_disbandreq;
-	t_server_w3xp_clan_disbandreply     server_w3xp_clan_disbandreply;
-	t_client_w3xp_clan_motdchg          client_w3xp_clan_motdchg;
-	t_client_w3xp_clan_motdreq          client_w3xp_clan_motdreq;
-	t_server_w3xp_clan_motdreply        server_w3xp_clan_motdreply;
-	t_client_w3xp_clan_memberreq        client_w3xp_clan_memberreq;
-	t_server_w3xp_clan_memberreply      server_w3xp_clan_memberreply;
-    t_client_w3xp_clan_createreq        client_w3xp_clan_createreq;
-	t_server_w3xp_clan_createreply      server_w3xp_clan_createreply;
-	t_client_w3xp_clan_createinvitereq  client_w3xp_clan_createinvitereq;
-    t_server_w3xp_clan_createinvitereply    server_w3xp_clan_createinvitereply;
-	t_server_w3xp_clan_createinvitereq  server_w3xp_clan_createinvitereq;
-    t_client_w3xp_clan_createinvitereply    client_w3xp_clan_createinvitereply;
-    t_server_w3xp_clan_clanack			    server_w3xp_clan_clanack;
-    t_server_w3xp_clan_memberchangeack      server_w3xp_clan_memberchangeack;
-    t_client_w3xp_clan_memberchangereq      client_w3xp_clan_memberchangereq;
-    t_server_w3xp_clan_memberchangereply    server_w3xp_clan_memberchangereply;
-    t_client_w3xp_clan_memberdelreq         client_w3xp_clan_memberdelreq;
-    t_server_w3xp_clan_memberdelreply       server_w3xp_clan_memberdelreply;
-    t_client_w3xp_clan_membernewchiefreq    client_w3xp_clan_membernewchiefreq;
-    t_server_w3xp_clan_membernewchiefreply  server_w3xp_clan_membernewchiefreply;
-    t_server_w3xp_clan_clanleaveack         server_w3xp_clan_clanleaveack;
-    t_server_w3xp_clan_memberleaveack       server_w3xp_clan_memberleaveack;
+	t_client_w3xp_clan_invitereq		client_w3xp_clan_invitereq;
+    	t_server_w3xp_clan_invitereply		server_w3xp_clan_invitereply;
+	t_server_w3xp_clan_invitereq		server_w3xp_clan_invitereq;
+	t_client_w3xp_clan_invitereply		client_w3xp_clan_invitereply;
+	t_client_w3xp_clan_disbandreq		client_w3xp_clan_disbandreq;
+	t_server_w3xp_clan_disbandreply		server_w3xp_clan_disbandreply;
+	t_client_w3xp_clan_motdchg		client_w3xp_clan_motdchg;
+	t_client_w3xp_clan_motdreq		client_w3xp_clan_motdreq;
+	t_server_w3xp_clan_motdreply		server_w3xp_clan_motdreply;
+	t_client_w3xp_clanmemberlist_req	client_w3xp_clanmemberlist_req;
+	t_server_w3xp_clanmemberlist_reply	server_w3xp_clanmemberlist_reply;
+	t_client_w3xp_clan_createreq		client_w3xp_clan_createreq;
+	t_server_w3xp_clan_createreply		server_w3xp_clan_createreply;
+	t_client_w3xp_clan_createinvitereq	client_w3xp_clan_createinvitereq;
+	t_server_w3xp_clan_createinvitereply	server_w3xp_clan_createinvitereply;
+	t_server_w3xp_clan_createinvitereq	server_w3xp_clan_createinvitereq;
+	t_client_w3xp_clan_createinvitereply	client_w3xp_clan_createinvitereply;
+	t_server_w3xp_clan_clanack		server_w3xp_clan_clanack;
+	t_server_w3xp_clanmemberupdate          server_w3xp_clanmemberupdate;
+	t_client_w3xp_clanmember_rankupdate_req   client_w3xp_clanmember_rankupdate_req;
+	t_server_w3xp_clanmember_rankupdate_reply server_w3xp_clanmember_rankupdate_reply;
+	t_client_w3xp_clanmember_remove_req     client_w3xp_clanmember_remove_req;
+	t_server_w3xp_clanmember_remove_reply   server_w3xp_clanmember_remove_reply;
+	t_client_w3xp_clan_membernewchiefreq    client_w3xp_clan_membernewchiefreq;
+	t_server_w3xp_clan_membernewchiefreply  server_w3xp_clan_membernewchiefreply;
+	t_server_w3xp_clanquitnotify            server_w3xp_clanquitnotify;
+	t_server_w3xp_clanmember_removed_notify server_w3xp_clanmember_removed_notify;
 
 	t_server_findanongame_iconreply		server_findanongame_iconreply;
 	t_client_changeclient			client_changeclient;
-	/* restructure anongame [Omega] */
 	t_client_anongame			client_anongame;
 	t_server_anongame_search_reply		server_anongame_search_reply;
 	t_client_anongame_tournament_request	client_anongame_tournament_request;
 	t_server_anongame_tournament_reply	server_anongame_tournament_reply;
 
-    /* new packets supporting D2 1.10 & War3 1.13 */
-    t_client_setemailreply	   client_setemailreq;
-	t_server_setemailreq	   server_setemailreply;
-	t_client_getpasswordreq	   client_getpasswordreq;
-	t_client_changeemailreq	   client_changeemailreq;
-	t_client_crashdump	   client_crashdump;
+	t_client_setemailreply			client_setemailreq;
+	t_server_setemailreq			server_setemailreply;
+	t_client_getpasswordreq			client_getpasswordreq;
+	t_client_changeemailreq			client_changeemailreq;
+	t_client_crashdump			client_crashdump;
+	t_client_claninforeq			client_claninforeq;
+	t_server_claninforeply			server_claninforeply;
+	t_client_findanongame_profile_clan	client_findanongame_profile_clan;
+	t_server_findanongame_profile_clan	server_findanongame_profile_clan;
 	} u;
 } t_packet;
 
@@ -400,12 +363,9 @@ typedef struct
 #ifndef INCLUDED_PACKET_PROTOS
 #define INCLUDED_PACKET_PROTOS
 
-#ifdef USE_CHECK_ALLOC
-extern t_packet * packet_create_real(t_packet_class class, char const * fn, unsigned int ln) ;
-# define packet_create(C) packet_create_real(C,__FILE__"{packet_create}",__LINE__)
-#else
+#include "common/lstr.h"
+
 extern t_packet * packet_create(t_packet_class class) ;
-#endif
 extern void packet_destroy(t_packet const * packet);
 extern t_packet * packet_add_ref(t_packet * packet);
 extern void packet_del_ref(t_packet * packet);
@@ -422,6 +382,7 @@ extern unsigned int packet_get_flags(t_packet const * packet);
 extern int packet_set_flags(t_packet * packet, unsigned int flags);
 extern int packet_append_string(t_packet * packet, char const * str);
 extern int packet_append_ntstring(t_packet * packet, char const * str);
+extern int packet_append_lstr(t_packet * packet, t_lstr *lstr);
 extern int packet_append_data(t_packet * packet, void const * data, unsigned int len);
 extern void const * packet_get_raw_data_const(t_packet const * packet, unsigned int offset);
 extern void * packet_get_raw_data(t_packet * packet, unsigned int offset);
