@@ -161,7 +161,10 @@
 /* Added by Soar */
 /* time limit for new member as newer(whom cannot be promoted) in clan, (hrs) */
 #define CLAN_NEWER_TIME     168
-#define CLAN_MAX_MEMBERS    50
+#define CLAN_DEFAULT_MAX_MEMBERS    50
+/* hardcoded limits in the client */
+#define CLAN_MIN_MEMBERS 10
+#define CLAN_MAX_MEMBERS 100
 
 /* moved from account.h */
 #define MAX_FRIENDS 20
