@@ -66,6 +66,9 @@
 #  include <time.h>
 # endif
 #endif
+#ifdef HAVE_ASSERT_H
+# include <assert.h>
+#endif
 
 #include "compat/psock.h"
 #include "compat/strcasecmp.h"
@@ -81,6 +84,7 @@
 #include "handle_init.h"
 #include "handle_bnetd.h"
 #include "d2charfile.h"
+#include "common/fdwatch.h"
 #include "common/addr.h"
 #include "common/introtate.h"
 #include "common/network.h"
@@ -88,14 +92,16 @@
 #include "common/hashtable.h"
 #include "common/queue.h"
 #include "common/eventlog.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 static t_hashtable 	* connlist_head=NULL;
 static t_hashtable	* conn_charname_list_head=NULL;
+static t_list		* connlist_dead=NULL;
 static unsigned int	total_connection=0;
 
 static int conn_handle_connecting(t_connection * c);
-static int conn_create_packet(t_connection * c);
+static t_packet * conn_create_packet(t_connection * c);
 static int conn_handle_packet(t_connection * c, t_packet * packet);
 static int conn_handle_read(t_connection * c);
 static int conn_handle_write(t_connection * c);
@@ -142,10 +148,19 @@ extern int d2cs_connlist_create(void)
 extern int d2cs_connlist_destroy(void)
 {
 	t_connection 	* c;
+	t_elem		* curr;
 	
+
+        if (connlist_dead) {
+                d2cs_connlist_reap();
+                if (list_destroy(connlist_dead))
+                        eventlog(eventlog_level_error,__FUNCTION__,"error destroy conndead list");
+                connlist_dead = NULL;
+        }
+
 	BEGIN_HASHTABLE_TRAVERSE_DATA(connlist_head, c)
 	{
-		d2cs_conn_destroy(c);
+		d2cs_conn_destroy(c,&curr);
 	}
 	END_HASHTABLE_TRAVERSE_DATA()
 
@@ -163,6 +178,20 @@ extern int d2cs_connlist_destroy(void)
 	return 0;
 }
 
+extern int d2cs_connlist_reap(void)
+{
+	t_connection 	* c;
+	
+	if (!connlist_dead) return 0;
+	BEGIN_LIST_TRAVERSE_DATA(connlist_dead, c)
+	{
+		d2cs_conn_destroy(c,&curr_elem_);
+	}
+	END_LIST_TRAVERSE_DATA()
+
+	return 0;
+}
+
 extern int conn_check_multilogin(t_connection const * c,char const * charname)
 {
 	t_connection * conn;
@@ -220,7 +249,7 @@ extern t_connection * d2cs_connlist_find_connection_by_charname(char const * cha
 	return NULL;
 }
 
-static int conn_create_packet(t_connection * c)
+static t_packet * conn_create_packet(t_connection * c)
 {
 	t_packet	* packet;
 
@@ -231,15 +260,14 @@ static int conn_create_packet(t_connection * c)
 		CASE(conn_class_bnetd, packet=packet_create(packet_class_d2cs_bnetd));
 		default:
 			eventlog(eventlog_level_error,__FUNCTION__,"got bad connection class %d",c->class);
-			return -1;
+			return NULL;
 	}
 	if (!packet) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error create packet");
-		return 0;
+		return NULL;
 	}
-	queue_push_packet(&c->inqueue,packet);
-	packet_del_ref(packet);
-	return 0;
+	d2cs_conn_set_in_queue(c,packet);
+	return packet;
 }
 
 static int conn_handle_connecting(t_connection * c)
@@ -252,6 +280,11 @@ static int conn_handle_connecting(t_connection * c)
 	}
 	eventlog(eventlog_level_info,__FUNCTION__,"connected to %s",addr_num_to_addr_str(c->addr, c->port));
 	c->state=conn_state_init;
+        /* this is a kind of hack to not update fd but updating breaks kqueue
+         * and the clean fix would require a cache a userland copy of the kernel
+	 * kqueue fds, considering that it also doesnt brake anything else should do
+	 * for the moment 
+	fdwatch_update_fd(c->sock, fdwatch_type_read); */
 	switch (c->class) {
 		case conn_class_bnetd:
 			retval=handle_bnetd_init(c);
@@ -285,11 +318,12 @@ static int conn_handle_read(t_connection * c)
 	t_packet	* packet;
 	int		retval;
 
-	if (!queue_get_length((t_queue const * const *)&c->inqueue)) {
-		if (conn_create_packet(c)<0) return -1;
+	packet = d2cs_conn_get_in_queue(c);
+	if (!packet) {
+		packet = conn_create_packet(c);
+		if (!packet) return -1;
 		c->insize=0;
 	}
-	if (!(packet=queue_peek_packet((t_queue const * const *)&c->inqueue))) return 0;
 
 	switch (net_recv_packet(c->sock,packet,&c->insize)) {
 		case -1:
@@ -300,7 +334,7 @@ static int conn_handle_read(t_connection * c)
 			break;
 		case 1:
 			c->insize=0;
-			packet=queue_pull_packet(&c->inqueue);
+			d2cs_conn_set_in_queue(c,NULL);
 			retval=conn_handle_packet(c,packet);
 			packet_del_ref(packet);
 			break;
@@ -320,7 +354,7 @@ static int conn_handle_write(t_connection * c)
 		return conn_handle_connecting(c);
 	}
 
-	if (!(packet=queue_peek_packet((t_queue const * const *)&c->outqueue))) return 0;
+	if (!(packet=conn_peek_outqueue(c))) return 0;
 
 	switch (net_send_packet(c->sock, packet, &c->outsize)) {
 		case -1:
@@ -331,7 +365,7 @@ static int conn_handle_write(t_connection * c)
 			break;
 		case 1:
 			c->outsize=0;
-			packet=queue_pull_packet(&c->outqueue);
+			packet=conn_pull_outqueue(c);
 			packet_del_ref(packet);
 			retval=0;
 			break;
@@ -400,13 +434,11 @@ extern t_connection * d2cs_conn_create(int sock, unsigned int local_addr, unsign
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad socket");
 		return NULL;
 	}
-	if (!(c=malloc(sizeof(t_connection)))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate connection");
-		return NULL;
-	}
+	c=xmalloc(sizeof(t_connection));
 	c->charname=NULL;
 	c->account=NULL;
 	c->sock=sock;
+	c->fdw_idx = -1;
 	c->socket_flag=0;
 	c->local_addr=local_addr;
 	c->local_port=local_port;
@@ -418,6 +450,7 @@ extern t_connection * d2cs_conn_create(int sock, unsigned int local_addr, unsign
 	c->outqueue=NULL;
 	c->inqueue=NULL;
 	c->outsize=0;
+	c->outsizep=0;
 	c->insize=0;
 	c->charinfo=NULL;
 	c->d2gs_id=0;
@@ -427,7 +460,7 @@ extern t_connection * d2cs_conn_create(int sock, unsigned int local_addr, unsign
 	c->bnetd_sessionnum=0;
 	c->charname_hash=0;
 	if (hashtable_insert_data(connlist_head, c, c->sessionnum_hash)<0) {
-		free(c);
+		xfree(c);
 		eventlog(eventlog_level_error,__FUNCTION__,"error add connection to list");
 		return NULL;
 	}
@@ -436,8 +469,9 @@ extern t_connection * d2cs_conn_create(int sock, unsigned int local_addr, unsign
 	return c;
 }
 
-extern int d2cs_conn_destroy(t_connection * c)
+extern int d2cs_conn_destroy(t_connection * c, t_elem ** curr)
 {
+	t_elem * elem;
 
 	ASSERT(c,-1);
 	if (c->state==conn_state_destroying) return 0;
@@ -453,20 +487,22 @@ extern int d2cs_conn_destroy(t_connection * c)
 		s2s_destroy(c);
 	}
 	if (c->gamequeue) {
-		gq_destroy(c->gamequeue);
+		gq_destroy(c->gamequeue,&elem);
 	}
-	if (c->account) free((void *)c->account);
-	if (c->charinfo) free((void *)c->charinfo);
+	if (c->account) xfree((void *)c->account);
+	if (c->charinfo) xfree((void *)c->charinfo);
 	if (c->charname) d2cs_conn_set_charname(c,NULL);
-	queue_clear(&c->inqueue);
+	if (c->inqueue) packet_del_ref(c->inqueue);
 	queue_clear(&c->outqueue);
 
+	if (connlist_dead) list_remove_data(connlist_dead, c, curr);
+	fdwatch_del_fd(c->fdw_idx);
 	psock_shutdown(c->sock,PSOCK_SHUT_RDWR);
 	psock_close(c->sock);
 
 	total_connection--;
 	eventlog(eventlog_level_info,__FUNCTION__,"[%d] closed connection %d (%d left)",c->sock,c->sessionnum,total_connection);
-	free(c);
+	xfree(c);
 	return 0;
 }
 
@@ -484,7 +520,20 @@ extern t_conn_state d2cs_conn_get_state(t_connection const * c)
 
 extern int d2cs_conn_set_state(t_connection * c, t_conn_state state)
 {
+	t_elem * curr;
+
 	ASSERT(c,-1);
+	/* special case for destroying connections, add them to connlist_dead list */
+	if (state == conn_state_destroy && c->state != conn_state_destroy) {
+	    if (!connlist_dead)
+	        connlist_dead = list_create();
+	    list_append_data(connlist_dead, c);
+	} else if (state != conn_state_destroy && c->state == conn_state_destroy) {
+	    if (list_remove_data(connlist_dead, c, &curr)) {
+		eventlog(eventlog_level_error, __FUNCTION__, "could not remove dead connection");
+		return -1;
+	    }
+	}
 	c->state=state;
 	return 0;
 }
@@ -502,12 +551,18 @@ extern int d2cs_conn_set_class(t_connection * c, t_conn_class class)
 	return 0;
 }
 
-extern t_queue * * d2cs_conn_get_in_queue(t_connection const * c)
+extern t_packet * d2cs_conn_get_in_queue(t_connection const * c)
 {
 	ASSERT(c,NULL);
-	return (t_queue * *)&c->inqueue;
+	return c->inqueue;
 }
-	
+
+extern void d2cs_conn_set_in_queue(t_connection * c, t_packet *packet)
+{
+	assert(c);
+	c->inqueue = packet;
+}
+
 extern unsigned int d2cs_conn_get_out_size(t_connection const * c)
 {
 	ASSERT(c,0);
@@ -526,6 +581,53 @@ extern unsigned int d2cs_conn_get_in_size(t_connection const * c)
 	return c->insize;
 }
 
+extern int conn_push_outqueue(t_connection * c, t_packet * packet)
+{
+    if (!c)
+    {
+        eventlog(eventlog_level_error, __FUNCTION__, "got NULL connection");
+        return -1;
+    }
+
+    if (!packet)
+    {
+        eventlog(eventlog_level_error, __FUNCTION__, "got NULL packet");
+        return -1;
+    }
+
+    queue_push_packet((t_queue * *)&c->outqueue, packet);
+    if (!c->outsizep++) fdwatch_update_fd(c->fdw_idx, fdwatch_type_read | fdwatch_type_write);
+
+    return 0;
+}
+
+extern t_packet * conn_peek_outqueue(t_connection * c)
+{
+    if (!c)
+    {
+        eventlog(eventlog_level_error, __FUNCTION__, "got NULL connection");
+        return NULL;
+    }
+
+    return queue_peek_packet((t_queue const * const *)&c->outqueue);
+}
+
+extern t_packet * conn_pull_outqueue(t_connection * c)
+{
+    if (!c)
+    {
+        eventlog(eventlog_level_error, __FUNCTION__, "got NULL connection");
+        return NULL;
+    }
+
+    if (c->outsizep) {
+        if (!(--c->outsizep)) fdwatch_update_fd(c->fdw_idx, fdwatch_type_read);
+        return queue_pull_packet((t_queue * *)&c->outqueue);
+    }
+
+    return NULL;
+}
+
 extern int conn_add_socket_flag(t_connection * c, unsigned int flag)
 {
 	ASSERT(c,-1);
@@ -567,19 +669,14 @@ extern int conn_process_packet(t_connection * c, t_packet * packet, t_packet_han
 
 extern int d2cs_conn_set_account(t_connection * c, char const * account)
 {
-	char const * temp;
-
 	ASSERT(c,-1);
 	if (!account) {
-		if (c->account) free((void *)c->account);
+		if (c->account) xfree((void *)c->account);
 		c->account=NULL;
 	}
-	if (!(temp=strdup(account))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate temp for account");
-		return -1;
-	}
-	if (c->account) free((void *)c->account);
-	c->account=temp;
+	if (c->account) xfree((void *)c->account);
+	c->account=xstrdup(account);
+
 	return 0;
 }
 
@@ -595,25 +692,22 @@ extern int d2cs_conn_set_charname(t_connection * c, char const * charname)
 
 	ASSERT(c,-1);
 	temp=NULL;
-	if (charname && !(temp=strdup(charname))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate temp for charname");
-		return -1;
-	}
+	if (charname) temp=xstrdup(charname);
 	if (c->charname) {
 		if (hashtable_remove_data(conn_charname_list_head,c,c->charname_hash) <0) {
 			eventlog(eventlog_level_error,__FUNCTION__,"error remove charname %s from list",charname);
-			if (temp) free((void *)temp);
+			if (temp) xfree((void *)temp);
 			return -1;
 		}
 		hashtable_purge(conn_charname_list_head);
-		free((void *)c->charname);
+		xfree((void *)c->charname);
 	}
 	if (charname) {
 		c->charname=temp;
 		c->charname_hash=conn_charname_hash(charname);
 		if (hashtable_insert_data(conn_charname_list_head,c,c->charname_hash) <0) {
 			eventlog(eventlog_level_error,__FUNCTION__,"error insert charname %s to list",charname);
-			free((void *)c->charname);
+			xfree((void *)c->charname);
 			c->charname=NULL;
 			return -1;
 		}
@@ -680,21 +774,15 @@ extern unsigned int conn_get_charinfo_class(t_connection const * c)
 
 extern int conn_set_charinfo(t_connection * c, t_d2charinfo_summary const * charinfo)
 {
-	t_d2charinfo_summary * temp;
-
 	ASSERT(c,-1);
 	if (!charinfo) {
-		if (c->charinfo) free((void *)c->charinfo);
+		if (c->charinfo) xfree((void *)c->charinfo);
 		c->charinfo=NULL;
 		return 0;
 	}
-	if (!(temp=malloc(sizeof(t_d2charinfo_summary)))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate temp for charinfo");
-		return -1;
-	}
-	if (c->charinfo) free((void *)c->charinfo);
-	memcpy(temp,charinfo,sizeof(t_d2charinfo_summary));
-	c->charinfo=temp;
+	if (c->charinfo) xfree((void *)c->charinfo);
+	c->charinfo=xmalloc(sizeof(t_d2charinfo_summary));
+	memcpy((void*)c->charinfo,charinfo,sizeof(t_d2charinfo_summary));
 	return 0;
 }
 
@@ -745,7 +833,13 @@ extern int conn_set_bnetd_sessionnum(t_connection * c, unsigned int sessionnum)
 
 extern unsigned int conn_get_bnetd_sessionnum(t_connection const * c)
 {
-	ASSERT(c,-1);
+	ASSERT(c,0);
 	return c->bnetd_sessionnum;
 }
 
+extern int conn_add_fd(t_connection * c, t_fdwatch_type rw, fdwatch_handler handler)
+{
+	assert(c);
+	c->fdw_idx = fdwatch_add_fd(c->sock,rw,handler,c);
+	return c->fdw_idx;
+}
