@@ -51,6 +51,7 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
+#include "compat/access.h"
 #ifdef TIME_WITH_SYS_TIME
 # include <time.h>
 # include <sys/time.h>
@@ -67,16 +68,16 @@
 
 #include "d2charfile.h"
 #include "prefs.h"
-#include "xstring.h"
+#include "common/xstring.h"
 #include "common/bn_type.h"
 #include "common/eventlog.h"
+#include "common/d2char_checksum.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 static int d2charsave_init(void * buffer,char const * charname,unsigned char class,unsigned short status);
 static int d2charinfo_init(t_d2charinfo_file * chardata, char const * account, char const * charname,
 				unsigned char class, unsigned short status);
-static int d2charsave_checksum(unsigned char const * data, unsigned int len, unsigned int offset);
-
 
 static int d2charsave_init(void * buffer,char const * charname,unsigned char class,unsigned short status)
 {
@@ -195,24 +196,16 @@ extern int d2char_create(char const * account, char const * charname, unsigned c
 		return -1;
 	}
 
-	if (!(savefile=malloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for file");
-		return -1;
-	}
+	savefile=xmalloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1);
 	d2char_get_savefile_name(savefile,charname);
 	if ((fp=fopen(savefile,"rb"))) {
 		eventlog(eventlog_level_warn,__FUNCTION__,"character save file \"%s\" for \"%s\" already exist",savefile,charname);
 		fclose(fp);
-		free(savefile);
+		xfree(savefile);
 		return -1;
 	}
 	
-	if (!(infofile=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charinfo file");
-		free(savefile);
-		return -1;
-	}
-
+	infofile=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
 	d2char_get_infofile_name(infofile,account,charname);
 
 	now = time(NULL);
@@ -225,22 +218,22 @@ extern int d2char_create(char const * account, char const * charname, unsigned c
 
 	if (file_write(infofile,&chardata,sizeof(chardata))<0) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error writing info file \"%s\"",infofile);
-		unlink(infofile);
-		free(infofile);
-		free(savefile);
+		remove(infofile);
+		xfree(infofile);
+		xfree(savefile);
 		return -1;
 	}
 
 	if (file_write(savefile,buffer,size)<0) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error writing save file \"%s\"",savefile);
-		unlink(infofile);
-		unlink(savefile);
-		free(savefile);
-		free(infofile);
+		remove(infofile);
+		remove(savefile);
+		xfree(savefile);
+		xfree(infofile);
 		return -1;
 	}
-	free(savefile);
-	free(infofile);
+	xfree(savefile);
+	xfree(infofile);
 	eventlog(eventlog_level_info,__FUNCTION__,"character %s(*%s) class %d status 0x%X created",charname,account,class,status);
 	return 0;
 }
@@ -253,13 +246,10 @@ extern int d2char_find(char const * account, char const * charname)
 
 	ASSERT(account,-1);
 	ASSERT(charname,-1);
-	if (!(file=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charinfo file");
-		return -1;
-	}
+	file=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
 	d2char_get_infofile_name(file,account,charname);
 	fp=fopen(file,"rb");
-	free(file);
+	xfree(file);
 	if (fp) {
 		fclose(fp);
 		return 0;
@@ -307,19 +297,16 @@ extern int d2char_convert(char const * account, char const * charname)
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad account name \"%s\"",account);
 		return -1;
 	}
-	if (!(file=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charinfo file");
-		return -1;
-	}
+	file=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
 	d2char_get_infofile_name(file,account,charname);
 	if (!(fp=fopen(file,"rb+"))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"unable to open charinfo file \"%s\" for reading and writing (fopen: %s)",file,strerror(errno));
-		free(file);
+		eventlog(eventlog_level_error,__FUNCTION__,"unable to open charinfo file \"%s\" for reading and writing (fopen: %s)",file,pstrerror(errno));
+		xfree(file);
 		return -1;
 	}
-	free(file);
+	xfree(file);
 	if (fread(&charinfo,1,sizeof(charinfo),fp)!=sizeof(charinfo)) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error reading charinfo file for character \"%s\" (fread: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error reading charinfo file for character \"%s\" (fread: %s)",charname,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
@@ -333,29 +320,26 @@ extern int d2char_convert(char const * account, char const * charname)
 	
 	fseek(fp,0,SEEK_SET); /* FIXME: check return */
 	if (fwrite(&charinfo,1,sizeof(charinfo),fp)!=sizeof(charinfo)) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error writing charinfo file for character \"%s\" (fwrite: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error writing charinfo file for character \"%s\" (fwrite: %s)",charname,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
 	if (fclose(fp)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not close charinfo file for character \"%s\" after writing (fclose: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not close charinfo file for character \"%s\" after writing (fclose: %s)",charname,pstrerror(errno));
 		return -1;
 	}
 	
-	if (!(file=malloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charsave file");
-		return -1;
-	}
+	file=xmalloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1);
 	d2char_get_savefile_name(file,charname);
 	if (!(fp=fopen(file,"rb+"))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not open charsave file \"%s\" for reading and writing (fopen: %s)",file,strerror(errno));
-		free(file);
+		eventlog(eventlog_level_error,__FUNCTION__,"could not open charsave file \"%s\" for reading and writing (fopen: %s)",file,pstrerror(errno));
+		xfree(file);
 		return -1;
 	}
-	free(file);
+	xfree(file);
 	size=fread(buffer,1,sizeof(buffer),fp);
 	if (!feof(fp)) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error reading charsave file for character \"%s\" (fread: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error reading charsave file for character \"%s\" (fread: %s)",charname,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
@@ -374,12 +358,12 @@ extern int d2char_convert(char const * account, char const * charname)
 	}
 	fseek(fp,0,SEEK_SET); /* FIXME: check return */
 	if (fwrite(buffer,1,size,fp)!=size) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error writing charsave file for character %s (fwrite: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error writing charsave file for character %s (fwrite: %s)",charname,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
 	if (fclose(fp)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not close charsave file for character \"%s\" after writing (fclose: %s)",charname,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not close charsave file for character \"%s\" after writing (fclose: %s)",charname,pstrerror(errno));
 		return -1;
 	}
 	eventlog(eventlog_level_info,__FUNCTION__,"character %s(*%s) converted to expansion",charname,account);
@@ -401,27 +385,45 @@ extern int d2char_delete(char const * account, char const * charname)
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad account name \"%s\"",account);
 		return -1;
 	}
-	if (!(file=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charinfo file");
-		return -1;
-	}
+	
+	/* charsave file */
+	file=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
 	d2char_get_infofile_name(file,account,charname);
-	if (unlink(file)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"failed to unlink charinfo file \"%s\" (unlink: %s)",file,strerror(errno));
-		free(file);
-		return -1;
-	}
-	free(file);
-
-	if (!(file=malloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charsave file");
+	if (remove(file)<0) {
+		eventlog(eventlog_level_error,__FUNCTION__,"failed to delete charinfo file \"%s\" (remove: %s)",file,pstrerror(errno));
+		xfree(file);
 		return -1;
 	}
+	xfree(file);
+	
+	/* charinfo file */
+	file=xmalloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1);
 	d2char_get_savefile_name(file,charname);
-	if (unlink(file)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"failed to unlink charsave file \"%s\" (unlink: %s)",file,strerror(errno));
+	if (remove(file)<0) {
+		eventlog(eventlog_level_error,__FUNCTION__,"failed to delete charsave file \"%s\" (remove: %s)",file,pstrerror(errno));
 	}
-	free(file);
+	xfree(file);
+	
+	/* bak charsave file */
+	file=xmalloc(strlen(prefs_get_bak_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
+	d2char_get_bak_infofile_name(file,account,charname);
+	if (access(file, F_OK) == 0) {
+	    if (remove(file)<0) {
+		eventlog(eventlog_level_error,__FUNCTION__,"failed to delete bak charinfo file \"%s\" (remove: %s)",file,pstrerror(errno));
+	    }
+	}
+	xfree(file);
+	
+	/* bak charinfo file */
+	file=xmalloc(strlen(prefs_get_bak_charsave_dir())+1+strlen(charname)+1);
+	d2char_get_bak_savefile_name(file,charname);
+	if (access(file, F_OK) == 0) {
+	    if (remove(file)<0) {
+		eventlog(eventlog_level_error,__FUNCTION__,"failed to delete bak charsave file \"%s\" (remove: %s)",file,pstrerror(errno));
+	    }
+	}
+	xfree(file);
+	
 	eventlog(eventlog_level_info,__FUNCTION__,"character %s(*%s) deleted",charname,account);
 	return 0;
 }
@@ -459,30 +461,27 @@ extern int d2charinfo_load(char const * account, char const * charname, t_d2char
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad account name \"%s\"",account);
 		return -1;
 	}
-	if (!(file=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for file");
-		return -1;
-	}
+	file=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1+strlen(charname)+1);
 	d2char_get_infofile_name(file,account,charname);
 	size=sizeof(t_d2charinfo_file);
 	if (file_read(file,data,&size)<0) {
 		eventlog(eventlog_level_error,__FUNCTION__,"error loading character file %s",file);
-		free(file);
+		xfree(file);
 		return -1;
 	}
 	if (size!=sizeof(t_d2charinfo_file)) {
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad charinfo file %s (length %d)",charname,size);
-		free(file);
+		xfree(file);
 		return -1;
 	}
 	d2char_portrait_init(&data->portrait);
 	if (d2charinfo_check(data) < 0) {
-		free(file);
+		xfree(file);
 		return -1;
 	}
 	if (!(charstatus_get_ladder(bn_int_get(data->summary.charstatus)))) {
 		bn_byte_set(&data->portrait.ladder, D2CHARINFO_PORTRAIT_PADBYTE);
-		free(file);
+		xfree(file);
 		return 0;
 	}
 	ladder_time = prefs_get_ladder_start_time();
@@ -499,10 +498,10 @@ extern int d2charinfo_load(char const * account, char const * charname, t_d2char
 		eventlog(eventlog_level_info,__FUNCTION__,"%s(*%s) was created in old ladder season, set to non-ladder", charname, account);
 		if (!(fp=fopen(file,"wb"))) {
 			eventlog(eventlog_level_error,__FUNCTION__,"charinfo file \"%s\" does not exist for account \"%s\"",file,account);
-			free(file);
+			xfree(file);
 			return 0;
 		}
-		free(file);
+		xfree(file);
 		charstatus = bn_int_get(data->summary.charstatus);
 		charstatus_set_ladder(charstatus, 0);
 		bn_int_set(&data->summary.charstatus, charstatus);
@@ -513,27 +512,24 @@ extern int d2charinfo_load(char const * account, char const * charname, t_d2char
 		bn_byte_set(&data->portrait.ladder, D2CHARINFO_PORTRAIT_PADBYTE);
 
 		if (fwrite(data,1,sizeof(*data),fp)!=sizeof(*data)) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error writing charinfo file for character \"%s\" (fwrite: %s)",charname,strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"error writing charinfo file for character \"%s\" (fwrite: %s)",charname,pstrerror(errno));
 			fclose(fp);
 			return 0;
 }
 		fclose(fp);
 
-		if (!(file=malloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for charsave file");
-			return -1;
-		}
+		file=xmalloc(strlen(prefs_get_charsave_dir())+1+strlen(charname)+1);
 		d2char_get_savefile_name(file,charname);
 
 		if (!(fp=fopen(file,"rb+"))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not open charsave file \"%s\" for reading and writing (fopen: %s)",file,strerror(errno));
-			free(file);
+			eventlog(eventlog_level_error,__FUNCTION__,"could not open charsave file \"%s\" for reading and writing (fopen: %s)",file,pstrerror(errno));
+			xfree(file);
 			return 0;
 		}
-		free(file);
+		xfree(file);
 		size=fread(buffer,1,sizeof(buffer),fp);
 		if (!feof(fp)) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error reading charsave file for character \"%s\" (fread: %s)",charname,strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"error reading charsave file for character \"%s\" (fread: %s)",charname,pstrerror(errno));
 			fclose(fp);
 			return 0;
 		}
@@ -553,14 +549,14 @@ extern int d2charinfo_load(char const * account, char const * charname, t_d2char
 		}
 		fseek(fp,0,SEEK_SET);
 		if (fwrite(buffer,1,size,fp)!=size) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error writing charsave file for character %s (fwrite: %s)",charname,strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"error writing charsave file for character %s (fwrite: %s)",charname,pstrerror(errno));
 			fclose(fp);
 			return 0;
 		}
 		fclose(fp);
 	} else {
 		bn_byte_set(&data->portrait.ladder, 1);
-		free(file);
+		xfree(file);
 	}
 	return 0;
 }
@@ -616,7 +612,7 @@ extern int d2char_check_charname(char const * name)
 	unsigned char	ch;
 	
 	if (!name) return -1;
-	if (!isalpha(name[0])) return -1;
+	if (!isalpha((int)name[0])) return -1;
 
 	for (i=1; i<=MAX_CHARNAME_LEN; i++) {
 		ch=name[i];
@@ -638,7 +634,7 @@ extern int d2char_check_acctname(char const * name)
 	unsigned char	ch;
 	
 	if (!name) return -1;
-	if (!isalnum(name[0])) return -1;
+	if (!isalnum((int)name[0])) return -1;
 
 	for (i=1; i<=MAX_CHARNAME_LEN; i++) {
 		ch=name[i];
@@ -666,6 +662,20 @@ extern int d2char_get_savefile_name(char * filename, char const * charname)
 }
 
 
+extern int d2char_get_bak_savefile_name(char * filename, char const * charname)
+{
+	char	tmpchar[MAX_CHARNAME_LEN];
+
+	ASSERT(filename,-1);
+	ASSERT(charname,-1);
+	strncpy(tmpchar,charname,sizeof(tmpchar));
+	tmpchar[sizeof(tmpchar)-1]='\0';
+	strtolower(tmpchar);
+	sprintf(filename,"%s/%s",prefs_get_bak_charsave_dir(),tmpchar);
+	return 0;
+}
+
+
 extern int d2char_get_infodir_name(char * filename, char const * account)
 {
 	char	tmpacct[MAX_ACCTNAME_LEN];
@@ -700,6 +710,27 @@ extern int d2char_get_infofile_name(char * filename, char const * account, char
 	return 0;
 }
 
+
+extern int d2char_get_bak_infofile_name(char * filename, char const * account, char const * charname)
+{
+	char	tmpchar[MAX_CHARNAME_LEN];
+	char	tmpacct[MAX_ACCTNAME_LEN];
+
+	ASSERT(filename,-1);
+	ASSERT(account,-1);
+	ASSERT(charname,-1);
+	strncpy(tmpchar,charname,sizeof(tmpchar));
+	tmpchar[sizeof(tmpchar)-1]='\0';
+	strtolower(tmpchar);
+
+	strncpy(tmpacct,account,sizeof(tmpacct));
+	tmpchar[sizeof(tmpacct)-1]='\0';
+	strtolower(tmpacct);
+	sprintf(filename,"%s/%s/%s",prefs_get_bak_charinfo_dir(),tmpacct,tmpchar);
+	return 0;
+}
+
+
 extern unsigned int d2charinfo_get_ladder(t_d2charinfo_summary const * charinfo)
 {
 	ASSERT(charinfo,0);
@@ -755,25 +786,6 @@ extern unsigned int d2charinfo_get_difficulty(t_d2charinfo_summary const * chari
 	return difficulty;
 }
 
-
-static int d2charsave_checksum(unsigned char const * data, unsigned int len,unsigned int offset)
-{
-	int		checksum;
-	unsigned int	i;
-	unsigned int	ch;
-
-	if (!data) return 0;
-	checksum=0;
-	for (i=0; i<len; i++) {
-		ch=data[i];
-		if (i>=offset && i<offset+sizeof(int)) ch=0;
-		ch+=(checksum<0);
-		checksum=2*checksum+ch;
-	}
-	return checksum;
-}
-
-
 /* those functions should move to util.c */
 extern int file_read(char const * filename, void * data, unsigned int * size)
 {
@@ -784,7 +796,7 @@ extern int file_read(char const * filename, void * data, unsigned int * size)
 	ASSERT(data,-1);
 	ASSERT(size,-1);
 	if (!(fp=fopen(filename,"rb"))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for reading (fopen: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for reading (fopen: %s)",filename,pstrerror(errno));
 		return -1;
 	}
 	
@@ -794,12 +806,12 @@ extern int file_read(char const * filename, void * data, unsigned int * size)
 	rewind(fp); /* FIXME: check return value */
 	
 	if (fread(data,1,n,fp)!=n) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error reading file \"%s\" (fread: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error reading file \"%s\" (fread: %s)",filename,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
 	if (fclose(fp)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not close file \"%s\" after reading (fclose: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not close file \"%s\" after reading (fclose: %s)",filename,pstrerror(errno));
 		return -1;
 	}
 	*size=n;
@@ -815,16 +827,16 @@ extern int file_write(char const * filename, void * data, unsigned int size)
 	ASSERT(data,-1);
 	ASSERT(size,-1);
 	if (!(fp=fopen(filename,"wb"))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for writing (fopen: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not open file \"%s\" for writing (fopen: %s)",filename,pstrerror(errno));
 		return -1;
 	}
 	if (fwrite(data,1,size,fp)!=size) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error writing file \"%s\" (fwrite: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"error writing file \"%s\" (fwrite: %s)",filename,pstrerror(errno));
 		fclose(fp);
 		return -1;
 	}
 	if (fclose(fp)<0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not close file \"%s\" after writing (fclose: %s)",filename,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"could not close file \"%s\" after writing (fclose: %s)",filename,pstrerror(errno));
 		return -1;
 	}
 	return 0;
