@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000,2001	Onlyer	(onlyer@263.net)
+ * Copyright (C) 2004		Olaf Freyer (aaron@cs.tu-berlin.de)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -84,8 +85,12 @@
 #include "common/queue.h"
 #include "common/packet.h"
 #include "common/eventlog.h"
+#include "d2charlist.h"
+#include "common/elist.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
+
 static int d2cs_send_client_ladder(t_connection * c, unsigned char type, unsigned short from);
 static unsigned int d2cs_try_joingame(t_connection const * c, t_game const * game, char const * gamepass);
 
@@ -177,7 +182,7 @@ static int on_client_loginreq(t_connection * c, t_packet * packet)
 				packet->u.client_d2cs_loginreq.secret_hash,
 				sizeof(bnpacket->u.d2cs_bnetd_accountloginreq.secret_hash));
 			packet_append_string(bnpacket,account);
-			queue_push_packet(d2cs_conn_get_out_queue(bnetd_conn()),bnpacket);
+			conn_push_outqueue(bnetd_conn(),bnpacket);
 		}
 		packet_del_ref(bnpacket);
 	}
@@ -207,16 +212,15 @@ static int on_client_createcharreq(t_connection * c, t_packet * packet)
 	class=bn_short_get(packet->u.client_d2cs_createcharreq.class);
 	status=bn_short_get(packet->u.client_d2cs_createcharreq.status);
 
-	if (!(path=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for path");
-		return 0;
-	}
+	path=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1);
 	d2char_get_infodir_name(path,account);
 	if (!(dir=p_opendir(path))) {
 	        eventlog(eventlog_level_info,__FUNCTION__,"(*%s) charinfo directory do not exist, building it",account);
 		p_mkdir(path,S_IRWXU);
 	}
-		free(path);
+	else
+	  p_closedir(dir);
+	xfree(path);
 
 	if (d2char_create(account,charname,class,status)<0) {
 		eventlog(eventlog_level_warn,__FUNCTION__,"error create character %s for account %s",charname,account);
@@ -237,7 +241,7 @@ static int on_client_createcharreq(t_connection * c, t_packet * packet)
 					conn_get_bnetd_sessionnum(c));
 				packet_append_string(bnpacket,charname);
 				packet_append_string(bnpacket,(char const *)&data.portrait);
-				queue_push_packet(d2cs_conn_get_out_queue(bnetd_conn()),bnpacket);
+				conn_push_outqueue(bnetd_conn(),bnpacket);
 			}
 			packet_del_ref(bnpacket);
 		}
@@ -247,7 +251,7 @@ static int on_client_createcharreq(t_connection * c, t_packet * packet)
 		packet_set_size(rpacket,sizeof(t_d2cs_client_createcharreply));
 		packet_set_type(rpacket,D2CS_CLIENT_CREATECHARREPLY);
 		bn_int_set(&rpacket->u.d2cs_client_createcharreply.reply,reply);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	return 0;
@@ -265,8 +269,9 @@ static int on_client_creategamereq(t_connection * c, t_packet * packet)
 	unsigned int	leveldiff, maxchar, difficulty, expansion, hardcore, ladder;
 	unsigned int	seqno, reply;
 	unsigned int	pos;
+	t_elem		* elem;
 
-	pos=sizeof(t_client_creategamereq);
+	pos=sizeof(t_client_d2cs_creategamereq);
 	if (!(gamename=packet_get_str_const(packet,pos,MAX_GAMENAME_LEN))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"got bad game name");
 		return -1;
@@ -308,7 +313,7 @@ static int on_client_creategamereq(t_connection * c, t_packet * packet)
 		if (gq) {
 			eventlog(eventlog_level_error,__FUNCTION__,"client %d is already in game queue",d2cs_conn_get_sessionnum(c));
 			conn_set_gamequeue(c,NULL);
-			gq_destroy(gq);
+			gq_destroy(gq,&elem);
 			return 0;
 		} else if ((gq=gq_create(d2cs_conn_get_sessionnum(c), packet, gamename))) {
 			conn_set_gamequeue(c,gq);
@@ -345,7 +350,7 @@ static int on_client_creategamereq(t_connection * c, t_packet * packet)
 			bn_short_set(&rpacket->u.d2cs_client_creategamereply.u1,0);
 			bn_short_set(&rpacket->u.d2cs_client_creategamereply.gameid,0);
 			bn_int_set(&rpacket->u.d2cs_client_creategamereply.reply,reply);
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 	} else {
@@ -369,7 +374,7 @@ static int on_client_creategamereq(t_connection * c, t_packet * packet)
 				packet_append_string(gspacket,d2cs_conn_get_charname(c));
 				addr.s_addr = htonl(d2cs_conn_get_addr(c));
 				packet_append_string(gspacket,inet_ntoa(addr));
-				queue_push_packet(d2cs_conn_get_out_queue(d2gs_get_connection(gs)),gspacket);
+				conn_push_outqueue(d2gs_get_connection(gs),gspacket);
 			}
 			packet_del_ref(gspacket);
 			eventlog(eventlog_level_info,__FUNCTION__,"request create game %s on gs %d",gamename,d2gs_get_id(gs));
@@ -436,7 +441,7 @@ static int on_client_joingamereq(t_connection * c, t_packet * packet)
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.addr,0);
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.token,0);
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.reply,reply);
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 	} else {
@@ -457,7 +462,7 @@ static int on_client_joingamereq(t_connection * c, t_packet * packet)
 				packet_append_string(gspacket,account);
 				addr.s_addr = htonl(d2cs_conn_get_addr(c));
 				packet_append_string(gspacket,inet_ntoa(addr));
-				queue_push_packet(d2cs_conn_get_out_queue(d2gs_get_connection(gs)),gspacket);
+				conn_push_outqueue(d2gs_get_connection(gs),gspacket);
 			}
 			packet_del_ref(gspacket);
 			eventlog(eventlog_level_info,__FUNCTION__,"request join game %s for character %s on gs %d",gamename,
@@ -486,9 +491,9 @@ static int on_client_gamelistreq(t_connection * c, t_packet * packet)
 
 	elem=start_elem=gamelist_get_curr_elem();
 	if (!elem) elem=list_get_first_const(d2cs_gamelist());
-	else elem=elem_get_next_const(elem);
+	else elem=elem_get_next_const(d2cs_gamelist(),elem);
 
-	for (; elem != start_elem; elem=elem_get_next_const(elem)) {
+	for (; elem != start_elem; elem=elem_get_next_const(d2cs_gamelist(),elem)) {
 		if (!elem) {
 			elem=list_get_first_const(d2cs_gamelist());
 			if (elem == start_elem) break;
@@ -512,7 +517,7 @@ static int on_client_gamelistreq(t_connection * c, t_packet * packet)
 			bn_int_set(&rpacket->u.d2cs_client_gamelistreply.gameflag,game_get_gameflag(game));
 			packet_append_string(rpacket,d2cs_game_get_name(game));
 			packet_append_string(rpacket,game_get_desc(game));
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 			count++;
 			if (prefs_get_maxgamelist() && count>=prefs_get_maxgamelist()) break;
@@ -530,7 +535,7 @@ static int on_client_gamelistreq(t_connection * c, t_packet * packet)
 			packet_append_string(rpacket,"");
 			packet_append_string(rpacket,"");
 			packet_append_string(rpacket,"");
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 	}
@@ -583,7 +588,7 @@ static int on_client_gameinforeq(t_connection * c, t_packet * packet)
 		if (n!=game_get_currchar(game)) {
 			eventlog(eventlog_level_error,__FUNCTION__,"game %s character list corrupted",gamename);
 		}
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	return 0;
@@ -621,7 +626,7 @@ static int on_client_charloginreq(t_connection * c, t_packet * packet)
 			packet_set_size(rpacket,sizeof(t_d2cs_client_charloginreply));
 			packet_set_type(rpacket,D2CS_CLIENT_CHARLOGINREPLY);
 			bn_int_set(&rpacket->u.d2cs_client_charloginreply.reply, D2CS_CLIENT_CHARLOGINREPLY_EXPIRED);
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 		eventlog(eventlog_level_info,__FUNCTION__,"character %s(*%s) login rejected due to char expired",charname,account);
@@ -639,7 +644,7 @@ static int on_client_charloginreq(t_connection * c, t_packet * packet)
 				conn_get_bnetd_sessionnum(c));
 			packet_append_string(bnpacket,charname);
 			packet_append_string(bnpacket,(char const *)&data.portrait);
-			queue_push_packet(d2cs_conn_get_out_queue(bnetd_conn()),bnpacket);
+			conn_push_outqueue(bnetd_conn(),bnpacket);
 		}
 		packet_del_ref(bnpacket);
 	}
@@ -674,7 +679,7 @@ static int on_client_deletecharreq(t_connection * c, t_packet * packet)
 		packet_set_type(rpacket,D2CS_CLIENT_DELETECHARREPLY);
 		bn_short_set(&rpacket->u.d2cs_client_deletecharreply.u1,0);
 		bn_int_set(&rpacket->u.d2cs_client_deletecharreply.reply,reply);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	return 0;
@@ -748,7 +753,7 @@ static int d2cs_send_client_ladder(t_connection * c, unsigned char type, unsigne
 				curr_len -= 4;
 			}
 			bn_short_set(&rpacket->u.d2cs_client_ladderreply.curr_len,curr_len);
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 		cont_len += curr_len;
@@ -756,27 +761,39 @@ static int d2cs_send_client_ladder(t_connection * c, unsigned char type, unsigne
 	return 0;
 }
 
+#define MAX_MOTD_LENGTH 511
 static int on_client_motdreq(t_connection * c, t_packet * packet)
 {
 	t_packet	* rpacket;
+	char		* motd;
+	int		motd_len;
 
 	if (!packet)
 	    return -1;
 	
+	/* client will crash if motd is too long */
+	motd = xstrdup(prefs_get_motd());
+	motd_len = strlen(motd);
+	if (motd_len > MAX_MOTD_LENGTH) {
+		WARN2("motd length (%i) exceeds maximun value (%i)",motd_len,MAX_MOTD_LENGTH);
+		motd[MAX_MOTD_LENGTH]='\0';
+	}
 	if ((rpacket=packet_create(packet_class_d2cs))) {
 		packet_set_size(rpacket,sizeof(t_d2cs_client_motdreply));
 		packet_set_type(rpacket,D2CS_CLIENT_MOTDREPLY);
 		bn_byte_set(&rpacket->u.d2cs_client_motdreply.u1,0);
-		packet_append_string(rpacket,prefs_get_motd());
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		packet_append_string(rpacket,motd);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
+	xfree(motd);
 	return 0;
 }
 
 static int on_client_cancelcreategame(t_connection * c, t_packet * packet)
 {
 	t_gq	* gq;
+	t_elem	* elem;
 
 	if (!packet)
 	    return -1;
@@ -785,7 +802,7 @@ static int on_client_cancelcreategame(t_connection * c, t_packet * packet)
 		return 0;
 	}
 	conn_set_gamequeue(c,NULL);
-	gq_destroy(gq);
+	gq_destroy(gq,&elem);
 	return 0;
 }
 
@@ -820,7 +837,7 @@ static int on_client_charladderreq(t_connection * c, t_packet * packet)
 			bn_short_set(&rpacket->u.d2cs_client_ladderreply.total_len,0);
 			bn_short_set(&rpacket->u.d2cs_client_ladderreply.curr_len,0);
 			bn_short_set(&rpacket->u.d2cs_client_ladderreply.cont_len,0);
-			queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+			conn_push_outqueue(c,rpacket);
 			packet_del_ref(rpacket);
 		}
 		return 0;
@@ -838,8 +855,10 @@ static int on_client_charlistreq(t_connection * c, t_packet * packet)
 	char const		* account; 
 	char const		* charname; 
 	char			* path;
-	t_d2charinfo_file       charinfo;
+	t_d2charinfo_file       * charinfo;
 	unsigned int		n, maxchar;
+	t_elist			charlist_head;
+	char const		* charlist_sort_order;
 
 	if (!packet)
 	    return -1;
@@ -848,21 +867,17 @@ static int on_client_charlistreq(t_connection * c, t_packet * packet)
 		eventlog(eventlog_level_error,__FUNCTION__,"missing account for connection");
 		return -1;
 	}
-	if (!(path=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for path");
-		return 0;
-	}
+	path=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1);
+	charlist_sort_order = prefs_get_charlist_sort_order();
+
+	elist_init(&charlist_head);
+	
 	d2char_get_infodir_name(path,account);
 	maxchar=prefs_get_maxchar();
 	if ((rpacket=packet_create(packet_class_d2cs))) {
 		packet_set_size(rpacket,sizeof(t_d2cs_client_charlistreply));
 		packet_set_type(rpacket,D2CS_CLIENT_CHARLISTREPLY);
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply.u1,0);
-		if (prefs_allow_newchar()) {
-			bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,maxchar);
-		} else {
-			bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,0);
-		}
 		n=0;
 		if (!(dir=p_opendir(path))) {
 			eventlog(eventlog_level_info,__FUNCTION__,"(*%s) charinfo directory do not exist, building it",account);
@@ -870,23 +885,59 @@ static int on_client_charlistreq(t_connection * c, t_packet * packet)
 		} else {
 			while ((charname=p_readdir(dir))) {
 				if (charname[0]=='.') continue;
-				if (d2charinfo_load(account,charname,&charinfo)<0) {
+				charinfo = xmalloc(sizeof(t_d2charinfo_file));
+				if (d2charinfo_load(account,charname,charinfo)<0) {
 					eventlog(eventlog_level_error,__FUNCTION__,"error loading charinfo for %s(*%s)",charname,account);
+					xfree((void *)charinfo);
 					continue;
 				}
-				packet_append_string(rpacket,charinfo.header.charname);
-				packet_append_string(rpacket,(char *)&charinfo.portrait);
+				eventlog(eventlog_level_debug,__FUNCTION__,"adding char %s (*%s)", charname, account);
+				d2charlist_add_char(&charlist_head,charinfo,0);
 				n++;
 				if (n>=maxchar) break;
 			}
+			if (prefs_allow_newchar() && (n<maxchar)) {
+				bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,maxchar);
+			} else {
+				bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,0);
+			}
 			p_closedir(dir);
+			if (!strcmp(charlist_sort_order, "ASC"))
+			{
+			    t_elist * curr, * safe;
+			    t_d2charlist * ccharlist;
+			
+			    elist_for_each_safe(curr,&charlist_head,safe)
+			    {
+				ccharlist = elist_entry(curr,t_d2charlist,list);
+				packet_append_string(rpacket,ccharlist->charinfo->header.charname);
+				packet_append_string(rpacket,(char *)&ccharlist->charinfo->portrait);
+				xfree((void *)ccharlist->charinfo);
+				xfree((void *)ccharlist);
+			    }
+			}
+			else
+			{
+			    t_elist * curr, * safe;
+			    t_d2charlist * ccharlist;
+			
+			    elist_for_each_safe_rev(curr,&charlist_head,safe)
+			    {
+				ccharlist = elist_entry(curr,t_d2charlist,list);
+				packet_append_string(rpacket,ccharlist->charinfo->header.charname);
+				packet_append_string(rpacket,(char *)&ccharlist->charinfo->portrait);
+				xfree((void *)ccharlist->charinfo);
+				xfree((void *)ccharlist);
+
+			    }
+			}
 		}
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply.currchar,n);
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply.currchar2,n);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
-	free(path);
+	xfree(path);
 	return 0;
 }
 
@@ -897,11 +948,14 @@ static int on_client_charlistreq_110(t_connection * c, t_packet * packet)
 	char const		* account; 
 	char const		* charname; 
 	char			* path;
-	t_d2charinfo_file       charinfo;
-	unsigned int		n, maxchar;
 
-	t_d2cs_client_chardata	chardata;
+	t_d2charinfo_file       * charinfo;
+	unsigned int		n, maxchar;
+	t_elist			charlist_head;
+
 	unsigned int		exp_time;
+	unsigned int		curr_exp_time;
+	char const		* charlist_sort_order;
 
 	if (!packet)
 	    return -1;
@@ -910,21 +964,21 @@ static int on_client_charlistreq_110(t_connection * c, t_packet * packet)
 		eventlog(eventlog_level_error,__FUNCTION__,"missing account for connection");
 		return -1;
 	}
-	if (!(path=malloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for path");
-		return 0;
-	}
+	path=xmalloc(strlen(prefs_get_charinfo_dir())+1+strlen(account)+1);
+	charlist_sort_order = prefs_get_charlist_sort_order();
+	
+	elist_init(&charlist_head);
+	
 	d2char_get_infodir_name(path,account);
-	maxchar=prefs_get_maxchar();
+	if (prefs_allow_newchar()) 
+		maxchar=prefs_get_maxchar();
+	else
+		maxchar=0;
+	
 	if ((rpacket=packet_create(packet_class_d2cs))) {
 		packet_set_size(rpacket,sizeof(t_d2cs_client_charlistreply_110));
 		packet_set_type(rpacket,D2CS_CLIENT_CHARLISTREPLY_110);
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply_110.u1,0);
-		if (prefs_allow_newchar()) {
-			bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,maxchar);
-		} else {
-			bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,0);
-		}
 		n=0;
 		if (!(dir=p_opendir(path))) {
 			eventlog(eventlog_level_info,__FUNCTION__,"(*%s) charinfo directory do not exist, building it",account);
@@ -933,29 +987,71 @@ static int on_client_charlistreq_110(t_connection * c, t_packet * packet)
 			exp_time = prefs_get_char_expire_time();
 			while ((charname=p_readdir(dir))) {
 				if (charname[0]=='.') continue;
-				if (d2charinfo_load(account,charname,&charinfo)<0) {
+				charinfo = xmalloc(sizeof(t_d2charinfo_file));
+				if (d2charinfo_load(account,charname,charinfo)<0) {
 					eventlog(eventlog_level_error,__FUNCTION__,"error loading charinfo for %s(*%s)",charname,account);
+					xfree(charinfo);
 					continue;
 				}
 				if (exp_time) {
-					bn_int_set(&chardata.expire_time, bn_int_get(charinfo.header.last_time)+exp_time);
+					curr_exp_time = bn_int_get(charinfo->header.last_time)+exp_time;
 				} else {
-					bn_int_set(&chardata.expire_time, 0x7FFFFFFF);
+					curr_exp_time = 0x7FFFFFFF;
 				}
-				packet_append_data(rpacket, &chardata, sizeof(chardata));
-				packet_append_string(rpacket,charinfo.header.charname);
-				packet_append_string(rpacket,(char *)&charinfo.portrait);
+				eventlog(eventlog_level_debug,__FUNCTION__,"adding char %s (*%s)", charname, account);
+				d2charlist_add_char(&charlist_head,charinfo,curr_exp_time);
 				n++;
 				if (n>=maxchar) break;
 			}
+			if (n>=maxchar) 
+				maxchar = 0;
+			
 			p_closedir(dir);
+			if (!strcmp(charlist_sort_order, "ASC"))
+			{
+			    t_elist * curr, *safe;
+			    t_d2charlist * ccharlist;
+			
+			    elist_for_each_safe(curr,&charlist_head,safe)
+			    {
+			    	bn_int bn_exp_time;
+
+				ccharlist = elist_entry(curr,t_d2charlist,list);
+				bn_int_set(&bn_exp_time,ccharlist->expiration_time);
+				packet_append_data(rpacket,bn_exp_time,sizeof(bn_exp_time));
+				packet_append_string(rpacket,ccharlist->charinfo->header.charname);
+				packet_append_string(rpacket,(char *)&ccharlist->charinfo->portrait);
+				xfree((void *)ccharlist->charinfo);
+				xfree((void *)ccharlist);
+			    }
+			}
+			else
+			{
+			    t_elist * curr, *safe;
+			    t_d2charlist * ccharlist;
+			
+			    elist_for_each_safe_rev(curr,&charlist_head,safe)
+			    {
+			    	bn_int bn_exp_time;
+
+				ccharlist = elist_entry(curr,t_d2charlist,list);
+				bn_int_set(&bn_exp_time,ccharlist->expiration_time);
+				packet_append_data(rpacket,bn_exp_time,sizeof(bn_exp_time));
+				packet_append_string(rpacket,ccharlist->charinfo->header.charname);
+				packet_append_string(rpacket,(char *)&ccharlist->charinfo->portrait);
+				xfree((void *)ccharlist->charinfo);
+				xfree((void *)ccharlist);
+			    }
+			}
 		}
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply.currchar,n);
 		bn_short_set(&rpacket->u.d2cs_client_charlistreply.currchar2,n);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		bn_short_set(&rpacket->u.d2cs_client_charlistreply.maxchar,maxchar);
+
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
-	free(path);
+	xfree(path);
 	return 0;
 }
 
@@ -985,7 +1081,7 @@ static int on_client_convertcharreq(t_connection * c, t_packet * packet)
 		packet_set_size(rpacket,sizeof(t_d2cs_client_convertcharreply));
 		packet_set_type(rpacket,D2CS_CLIENT_CONVERTCHARREPLY);
 		bn_int_set(&rpacket->u.d2cs_client_convertcharreply.reply,reply);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	return 0;
@@ -1000,7 +1096,7 @@ extern int d2cs_send_client_creategamewait(t_connection * c, unsigned int positi
 		packet_set_size(packet,sizeof(t_d2cs_client_creategamewait));
 		packet_set_type(packet,D2CS_CLIENT_CREATEGAMEWAIT);
 		bn_int_set(&packet->u.d2cs_client_creategamewait.position,position);
-		queue_push_packet(d2cs_conn_get_out_queue(c),packet);
+		conn_push_outqueue(c,packet);
 		packet_del_ref(packet);
 	}
 	return 0;
