@@ -28,6 +28,12 @@
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h> /* needed to include netinet/in.h */
 #endif
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #ifdef HAVE_SYS_SOCKET_H
 # include <sys/socket.h>
 #endif
@@ -48,12 +54,13 @@
 #include "connection.h"
 #include "prefs.h"
 #include "d2cs_d2gs_protocol.h"
-#include "d2gstrans.h"
+#include "common/trans.h"
 #include "common/addr.h"
 #include "common/eventlog.h"
 #include "common/queue.h"
 #include "common/bn_type.h"
 #include "common/packet.h"
+#include "common/xalloc.h"
 #include "common/setup_after.h"
 
 DECLARE_PACKET_HANDLER(on_d2gs_authreply)
@@ -119,7 +126,7 @@ static void d2gs_send_init_info(t_d2gs * gs, t_connection * c)
 		packet_append_string(rpacket,"bogus_ac_checksum");
 //		packet_append_string(rpacket,prefs_get_d2gs_ac_string());
 		packet_append_string(rpacket,"bogus_ac_string");
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	return;
@@ -129,52 +136,47 @@ static void d2gs_send_server_conffile(t_d2gs *gs, t_connection *c)
 {
 	t_packet	*rpacket;
 	FILE		*fp;
-	char		*confs;
-	long		size;
+	char		buff[256];
+	int		rno;
+	struct stat	sfile;
 
 	/* open d2gs server config file */
-	if ((fp=fopen(prefs_get_d2gsconffile(), "r"))==NULL)
-		return;
+	fp = fopen(prefs_get_d2gsconffile(), "rb");
+	if (!fp) goto err;
+
 	/* get file size */
-	if (fseek(fp, 0L, SEEK_END)) {
-		fclose(fp);
-		eventlog(eventlog_level_debug,__FUNCTION__,"failed fseek()");
-		return;
+	if (fstat(fileno(fp),&sfile)) {
+		eventlog(eventlog_level_debug,__FUNCTION__,"failed fstat()");
+		goto err_fp;
 	}
-	if ((size = ftell(fp)) <= 0) {
-		fclose(fp);
-		eventlog(eventlog_level_error,__FUNCTION__,"failed ftell()");
-		return;
+
+	rpacket = packet_create(packet_class_d2cs);
+	packet_set_size(rpacket,sizeof(t_d2cs_d2gs_setconffile));
+	packet_set_type(rpacket,D2CS_D2GS_SETCONFFILE);
+	bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.h.seqno,0);
+	bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.size, sfile.st_size);
+	bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.reserved1, time(NULL));
+
+	while((rno = fread(buff,1,sizeof(buff),fp)) > 0) {
+		if (packet_append_data(rpacket,buff,rno) < 0) { /* file too big */
+		    eventlog(eventlog_level_debug,__FUNCTION__,"conffile too big for a single packet");
+		    goto err_packet;
+		}
 	}
-	fseek(fp, 0L, SEEK_SET);
-	/* read config file content */
-	confs = malloc(size+8);
-	if (confs==NULL) {
-		fclose(fp);
-		eventlog(eventlog_level_error,__FUNCTION__,"not enough memory");
-		return;
-	}
-	if (fread(confs, size, 1, fp) != 1) {
-		fclose(fp);
-		free(confs);
-		eventlog(eventlog_level_error,__FUNCTION__,"failed fread()");
-		return;
-	}
-	confs[size]='\0';
+
+	conn_push_outqueue(c,rpacket);
+	packet_del_ref(rpacket);
 	fclose(fp);
-	/* send the config within a packet */
-	if ((rpacket=packet_create(packet_class_d2gs))) {
-		packet_set_size(rpacket,sizeof(t_d2cs_d2gs_setconffile));
-		packet_set_type(rpacket,D2CS_D2GS_SETCONFFILE);
-		bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.h.seqno,0);
-		bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.size, size);
-		bn_int_set(&rpacket->u.d2cs_d2gs_setconffile.reserved1, time(NULL));
-		packet_append_string(rpacket,confs);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
-		packet_del_ref(rpacket);
-		eventlog(eventlog_level_info,__FUNCTION__,"send config file to d2gs %s", addr_num_to_ip_str(d2cs_conn_get_addr(c)));
-	}
-	free(confs);
+
+	return;
+
+err_packet:
+	packet_del_ref(rpacket);
+
+err_fp:
+	fclose(fp);
+
+err:
 	return;
 }
 
@@ -236,7 +238,7 @@ static int on_d2gs_authreply(t_connection * c, t_packet * packet)
 		packet_set_type(rpacket,D2CS_D2GS_AUTHREPLY);
 		bn_int_set(&rpacket->u.d2cs_d2gs_authreply.h.seqno,0);
 		bn_int_set(&rpacket->u.d2cs_d2gs_authreply.reply,reply);
-		queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+		conn_push_outqueue(c,rpacket);
 		packet_del_ref(rpacket);
 	}
 	
@@ -267,7 +269,7 @@ static int on_d2gs_setgsinfo(t_connection * c, t_packet * packet)
 	    bn_int_set(&rpacket->u.d2cs_d2gs_setgsinfo.h.seqno,0);
 	    bn_int_set(&rpacket->u.d2cs_d2gs_setgsinfo.maxgame,maxgame);
 	    bn_int_set(&rpacket->u.d2cs_d2gs_setgsinfo.gameflag,gameflag);
-	    queue_push_packet(d2cs_conn_get_out_queue(c),rpacket);
+	    conn_push_outqueue(c,rpacket);
 	    packet_del_ref(rpacket);
         }
 
@@ -291,6 +293,7 @@ static int on_d2gs_creategamereply(t_connection * c, t_packet * packet)
 	int		result;
 	int		reply;
 	int		seqno;
+	t_elem		* curr;
 
 	seqno=bn_int_get(packet->u.d2cs_d2gs.h.seqno);
 	if (!(sq=sqlist_find_sq(seqno))) {
@@ -299,17 +302,17 @@ static int on_d2gs_creategamereply(t_connection * c, t_packet * packet)
 	}
 	if (!(client=d2cs_connlist_find_connection_by_sessionnum(sq_get_clientid(sq)))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"client %d not found",sq_get_clientid(sq));
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 	if (!(game=gamelist_find_game_by_id(sq_get_gameid(sq)))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"game %d not found",sq_get_gameid(sq));
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 	if (!(opacket=sq_get_packet(sq))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"previous packet not found (seqno: %d)",seqno);
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 
@@ -321,7 +324,7 @@ static int on_d2gs_creategamereply(t_connection * c, t_packet * packet)
 		reply=D2CS_CLIENT_CREATEGAMEREPLY_SUCCEED;
 	} else {
 		eventlog(eventlog_level_warn,__FUNCTION__,"failed to create game %s on gs %d",d2cs_game_get_name(game),conn_get_d2gs_id(c));
-		game_destroy(game);
+		game_destroy(game,&curr);
 		reply=D2CS_CLIENT_CREATEGAMEREPLY_FAILED;
 	}
 
@@ -333,10 +336,10 @@ static int on_d2gs_creategamereply(t_connection * c, t_packet * packet)
 		bn_short_set(&rpacket->u.d2cs_client_creategamereply.gameid,1);
 		bn_short_set(&rpacket->u.d2cs_client_creategamereply.u1,1);
 		bn_int_set(&rpacket->u.d2cs_client_creategamereply.reply,reply);
-		queue_push_packet(d2cs_conn_get_out_queue(client),rpacket);
+		conn_push_outqueue(client,rpacket);
 		packet_del_ref(rpacket);
 	}
-	sq_destroy(sq);
+	sq_destroy(sq,&curr);
 	return 0;
 }
 
@@ -351,6 +354,8 @@ static int on_d2gs_joingamereply(t_connection * c, t_packet * packet)
 	int		reply;
 	int		seqno;
 	unsigned int	gsaddr;
+	t_elem		* curr;
+	unsigned short	gsport;
 			
 
 	seqno=bn_int_get(packet->u.d2cs_d2gs.h.seqno);
@@ -360,22 +365,22 @@ static int on_d2gs_joingamereply(t_connection * c, t_packet * packet)
 	}
 	if (!(client=d2cs_connlist_find_connection_by_sessionnum(sq_get_clientid(sq)))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"client %d not found",sq_get_clientid(sq));
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 	if (!(game=gamelist_find_game_by_id(sq_get_gameid(sq)))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"game %d not found",sq_get_gameid(sq));
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 	if (!(gs=game_get_d2gs(game))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"try join game without game server set");
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 	if (!(opacket=sq_get_packet(sq))) {
 		eventlog(eventlog_level_error,__FUNCTION__,"previous packet not found (seqno: %d)",seqno);
-		sq_destroy(sq);
+		sq_destroy(sq,&curr);
 		return 0;
 	}
 
@@ -416,7 +421,8 @@ static int on_d2gs_joingamereply(t_connection * c, t_packet * packet)
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.token,sq_get_gametoken(sq));
 
 			gsaddr = d2gs_get_ip(gs);
-			d2gstrans_net(d2cs_conn_get_addr(client),&gsaddr);
+			gsport = 4000;
+			trans_net(d2cs_conn_get_addr(client), &gsaddr, &gsport);
 			
 			if(d2gs_get_ip(gs)!=gsaddr)
 			{
@@ -431,10 +437,10 @@ static int on_d2gs_joingamereply(t_connection * c, t_packet * packet)
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.token,0);
 			bn_int_set(&rpacket->u.d2cs_client_joingamereply.addr,0);
 		}
-		queue_push_packet(d2cs_conn_get_out_queue(client),rpacket);
+		conn_push_outqueue(client,rpacket);
 		packet_del_ref(rpacket);
 	}
-	sq_destroy(sq);
+	sq_destroy(sq,&curr);
 	return 0;
 }
 
@@ -476,12 +482,13 @@ static int on_d2gs_updategameinfo(t_connection * c, t_packet * packet)
 static int on_d2gs_closegame(t_connection * c, t_packet * packet)
 {
 	t_game	* game;
+	t_elem  * curr;
 
 	if (!(game=gamelist_find_game_by_d2gs_and_id(conn_get_d2gs_id(c),
 		bn_int_get(packet->u.d2gs_d2cs_closegame.gameid)))) {
 		return 0;
 	}
-	game_destroy(game);
+	game_destroy(game, &curr);
 	return 0;
 }
 
@@ -503,7 +510,7 @@ extern int handle_d2gs_init(t_connection * c)
 		bn_int_set(&packet->u.d2cs_d2gs_authreq.signlen, 0);
 		packet_append_string(packet,prefs_get_realmname());
 //		packet_append_data(packet, sign, signlen);
-		queue_push_packet(d2cs_conn_get_out_queue(c),packet);
+		conn_push_outqueue(c,packet);
 		packet_del_ref(packet);
 	}
 	eventlog(eventlog_level_info,__FUNCTION__,"sent init packet to d2gs %d (sessionnum=%d)",conn_get_d2gs_id(c),d2cs_conn_get_sessionnum(c));
