@@ -18,6 +18,10 @@
 #ifndef INCLUDED_SERVER_H
 #define INCLUDED_SERVER_H
 
+#include "common/fdwatch.h"
+
 extern int d2cs_server_process(void);
+extern int d2cs_server_handle_tcp(void*,t_fdwatch_type);
+extern int d2cs_server_handle_accept(void*,t_fdwatch_type);
 
 #endif
