@@ -63,19 +63,17 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-
-#ifdef WIN32
-# include <io.h>
-#endif
-
+#include "compat/rename.h"
 #include "d2ladder.h"
 #include "prefs.h"
 #include "common/tag.h"
 #include "common/list.h"
 #include "common/eventlog.h"
 #include "compat/strncasecmp.h"
-#include "common/setup_after.h"
+#include "compat/strerror.h"
 #include "d2cs/d2charfile.h"
+#include "common/xalloc.h"
+#include "common/setup_after.h"
 
 char			* d2ladder_ladder_file = NULL;
 char			* d2ladder_backup_file = NULL;
@@ -98,7 +96,6 @@ int d2ladder_check(void);
 int d2ladder_readladder(void);
 
 int d2ladder_insert(t_d2ladder * d2ladder,t_d2ladder_info * pcharladderinfo);
-int d2ladder_find_char(t_d2ladder * d2ladder, t_d2ladder_info * info);
 int d2ladder_find_char_all(t_d2ladder * d2ladder, t_d2ladder_info * info);
 int d2ladder_find_pos(t_d2ladder * d2ladder, t_d2ladder_info * info);
 int d2ladder_update_info_and_pos(t_d2ladder * d2ladder, t_d2ladder_info * info, int oldpos, int newpos);
@@ -215,7 +212,7 @@ int d2ladder_find_pos(t_d2ladder * d2ladder, t_d2ladder_info * info)
 	
 	i=d2ladder->len;
 	while (i--) {
-		if (d2ladder->info[i].experience >= info->experience) {
+		if (d2ladder->info[i].experience > info->experience) {
 			if (strncasecmp(d2ladder->info[i].charname,info->charname, MAX_CHARNAME_LEN)) {
 				i++;
 			}
@@ -236,7 +233,7 @@ int d2ladder_insert(t_d2ladder * d2ladder,t_d2ladder_info * info)
 	/*
 	if (newpos<0 || newpos >= d2ladder->len) return 0;
 	*/
-	oldpos=d2ladder_find_char(d2ladder,info);
+	oldpos=d2ladder_find_char_all(d2ladder,info);
 	return d2ladder_update_info_and_pos(d2ladder,info,oldpos,newpos);
 }
 
@@ -255,22 +252,6 @@ int d2ladder_find_char_all(t_d2ladder * d2ladder, t_d2ladder_info * info)
 	return -1;
 }
 
-int d2ladder_find_char(t_d2ladder * d2ladder, t_d2ladder_info * info)
-{
-	int		i;
-	t_d2ladder_info * ladderdata;
-
-	if (!d2ladder || !info) return -1;
-	ladderdata=d2ladder->info;
-	if (!ladderdata) return -1;
-	i=d2ladder->len;
-	while (i--) {
-		if (ladderdata[i].level > info->level) return -1;
-		if (!strncasecmp(ladderdata[i].charname,info->charname,MAX_CHARNAME_LEN)) return i;
-	}
-	return -1;
-}
-
 int d2ladder_update_info_and_pos(t_d2ladder * d2ladder, t_d2ladder_info * info, int oldpos, int newpos)
 {
 	int	i;
@@ -317,15 +298,7 @@ int d2ladder_check(void)
 	if (!d2ladder_backup_file) return -1;
 	if(d2ladder_checksum_check()!=1) {
 		eventlog(eventlog_level_error,__FUNCTION__,"ladder file checksum error,try to use backup file");
-#ifdef WIN32
-		if (access(d2ladder_ladder_file, 0) == 0) {
-			if (remove(d2ladder_ladder_file)<0) {
-				eventlog(eventlog_level_error,__FUNCTION__,"could not delete ladder file \"%s\" (remove: %s)",d2ladder_ladder_file);
-				return -1;
-			}
-		}
-#endif
-		if (rename(d2ladder_backup_file,d2ladder_ladder_file)==-1) {
+		if (p_rename(d2ladder_backup_file,d2ladder_ladder_file)==-1) {
 			eventlog(eventlog_level_error,__FUNCTION__,"error rename %s to %s", d2ladder_backup_file,d2ladder_ladder_file);
 		}
 		if(d2ladder_checksum_check()!=1) {
@@ -364,12 +337,10 @@ extern int d2dbs_d2ladder_init(void)
 {
 	d2ladder_change_count=0;
 	d2ladder_maxtype=0;
-	d2ladder_ladder_file=malloc(strlen(d2dbs_prefs_get_ladder_dir())+1+\
+	d2ladder_ladder_file=xmalloc(strlen(d2dbs_prefs_get_ladder_dir())+1+
 			  strlen(LADDER_FILE_PREFIX)+1+strlen(CLIENTTAG_DIABLO2DV)+1+10);
-	d2ladder_backup_file=malloc(strlen(d2dbs_prefs_get_ladder_dir())+1+\
+	d2ladder_backup_file=xmalloc(strlen(d2dbs_prefs_get_ladder_dir())+1+
 			  strlen(LADDER_BACKUP_PREFIX)+1+strlen(CLIENTTAG_DIABLO2DV)+1+10);
-	if (!d2ladder_ladder_file) return -1;
-	if (!d2ladder_backup_file) return -1;
 	sprintf(d2ladder_ladder_file,"%s/%s.%s",d2dbs_prefs_get_ladder_dir(),\
 		LADDER_FILE_PREFIX,CLIENTTAG_DIABLO2DV);
 
@@ -398,16 +369,10 @@ int d2ladderlist_init(void)
 
 
 	if (!d2ladder_ladder_file) return -1;
-	if (!(d2ladder_list=list_create())) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not create d2ladder_list");
-		return -1;
-	}
+	d2ladder_list=list_create();
 	d2ladder_maxtype=D2LADDER_MAXTYPE;
 	for (i=0;i<d2ladder_maxtype;i++) {
-		if (!(d2ladder=malloc(sizeof(t_d2ladder)))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"could not allocate d2ladder");
-			return -1;
-		}
+		d2ladder=xmalloc(sizeof(t_d2ladder));
 		d2ladder->type=i;
 		d2ladder->info=NULL;
 		d2ladder->len=0;
@@ -451,7 +416,7 @@ int d2ladder_readladder(void)
 
 	readlen=fread(&fileheader,1,sizeof(fileheader),fdladder);
 	if (readlen<=0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,strerror(errno));
+		eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,pstrerror(errno));
 		fclose(fdladder);
 		return -1;
 	}
@@ -472,15 +437,11 @@ int d2ladder_readladder(void)
 		return -1;
 	}
 
-	if (!(lhead=malloc(blocksize))) {
-		eventlog(eventlog_level_error,__FUNCTION__,"could not allocate memory for lhead");
-		fclose(fdladder);
-		return -1;
-    	}
+	lhead=xmalloc(blocksize);
 	readlen=fread(lhead,1,d2ladder_maxtype*sizeof(*lhead),fdladder);
 	if (readlen<=0) {
-		eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,strerror(errno));
-		free(lhead);
+		eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,pstrerror(errno));
+		xfree(lhead);
 		fclose(fdladder);
 		return -1;
 	}
@@ -492,7 +453,7 @@ int d2ladder_readladder(void)
 	}
 	if (leftsize < blocksize ) {
 		eventlog(eventlog_level_error,__FUNCTION__,"file size error");
-		free(lhead);
+		xfree(lhead);
 		fclose(fdladder);
 		return -1;
 	}
@@ -505,22 +466,15 @@ int d2ladder_readladder(void)
 			eventlog(eventlog_level_error,__FUNCTION__,"could not find ladder type %d",laddertype);
 			continue;
 		}
-		if (!(ldata=malloc(number*sizeof(*ldata)))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for ldata");
-			continue;
-		}
-		if (!(info=malloc(number * sizeof(*info)))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for info");
-			free(ldata);
-			continue;
-		}
+		ldata=xmalloc(number*sizeof(*ldata));
+		info=xmalloc(number * sizeof(*info));
 		memset(info,0,number * sizeof(*info));
 		fseek(fdladder,bn_int_get(lhead[laddertype].offset),SEEK_SET);
 		readlen=fread(ldata,1,number*sizeof(*ldata),fdladder);
 		if (readlen<=0) {
-			eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,strerror(errno));
-			free(ldata);
-			free(info);
+			eventlog(eventlog_level_error,__FUNCTION__,"file %s read error(read:%s)",d2ladder_ladder_file,pstrerror(errno));
+			xfree(ldata);
+			xfree(info);
 			continue;
 		}
 		d2ladder->info=info;
@@ -538,11 +492,11 @@ int d2ladder_readladder(void)
 				d2ladder_change_count++;
 			}
 		}
-		free(ldata);
+		xfree(ldata);
 	}
 	leftsize-=blocksize;
 	
-	free(lhead);
+	xfree(lhead);
 	fclose(fdladder);
 	return 0;
 }
@@ -558,18 +512,18 @@ extern int d2dbs_d2ladder_destroy(void)
 		if(d2ladder)
 		{
 			if(d2ladder->info)
-				free(d2ladder->info);
+				xfree(d2ladder->info);
 			d2ladder->info=NULL;
 			d2ladder->len=0;
 		}
     	}
 	d2ladderlist_destroy();
 	if (d2ladder_ladder_file) {
-		free(d2ladder_ladder_file);
+		xfree(d2ladder_ladder_file);
 		d2ladder_ladder_file=NULL;
 	}
 	if (d2ladder_backup_file) {
-		free(d2ladder_backup_file);
+		xfree(d2ladder_backup_file);
 		d2ladder_backup_file=NULL;
 	}
 	return 0;
@@ -584,8 +538,8 @@ int d2ladderlist_destroy(void)
     	LIST_TRAVERSE(d2ladder_list,elem)
     	{
 		if (!(d2ladder=elem_get_data(elem))) continue;
-		free(d2ladder);
-		list_remove_elem(d2ladder_list,elem);
+		xfree(d2ladder);
+		list_remove_elem(d2ladder_list,&elem);
     	}
     	list_destroy(d2ladder_list);
 	return 0;
@@ -731,15 +685,7 @@ extern int d2ladder_saveladder(void)
 
 	if(d2ladder_checksum_check()==1) {
 		eventlog(eventlog_level_info,__FUNCTION__,"backup ladder file");
-#ifdef WIN32
-		if (access(d2ladder_backup_file, 0) == 0) {
-			if (remove(d2ladder_backup_file)<0) {
-				eventlog(eventlog_level_error,__FUNCTION__,"could not delete backup ladder file \"%s\" (remove: %s)",d2ladder_backup_file);
-				return -1;
-			}
-		}
-#endif
-		if (rename(d2ladder_ladder_file,d2ladder_backup_file)==-1) {
+		if (p_rename(d2ladder_ladder_file,d2ladder_backup_file)==-1) {
 			eventlog(eventlog_level_warn,__FUNCTION__,"error rename %s to %s", d2ladder_ladder_file, d2ladder_backup_file);
 		}
 	}
@@ -753,29 +699,22 @@ extern int d2ladder_saveladder(void)
 	// aaron: add extra output for XML ladder here --->
 	if (d2dbs_prefs_get_XML_output_ladder())
 	{
-	  XMLfilename = malloc(strlen(d2dbs_prefs_get_ladder_dir())+1+strlen(XMLname)+1);
-	  if (!XMLfilename) 
-	  { 
-	    eventlog(eventlog_level_error,__FUNCTION__,"could not alloc mem for XML ladder filename");
+	  XMLfilename = xmalloc(strlen(d2dbs_prefs_get_ladder_dir())+1+strlen(XMLname)+1);
+	  sprintf(XMLfilename,"%s/%s",d2dbs_prefs_get_ladder_dir(),XMLname);
+	  if (!(XMLfile = fopen(XMLfilename,"w")))
+	  {
+	      eventlog(eventlog_level_error,__FUNCTION__,"could not open XML ladder file for output");
 	  }
 	  else
 	  {
-	    sprintf(XMLfilename,"%s/%s",d2dbs_prefs_get_ladder_dir(),XMLname);
-	    if (!(XMLfile = fopen(XMLfilename,"w")))
-	    {
-	      eventlog(eventlog_level_error,__FUNCTION__,"could not open XML ladder file for output");
-	    }
-	    else
-	    {
-              d2ladder_print_XML(XMLfile);
-	      fclose(XMLfile);
-	      free(XMLfilename);
-	    }
+             d2ladder_print_XML(XMLfile);
+	     fclose(XMLfile);
+	     xfree(XMLfilename);
 	  }
 	}
-	
+
 	// <---
-	
+
 	bn_int_set(&fileheader.maxtype,d2ladder_maxtype);
 	bn_int_set(&fileheader.checksum,0);
 	fwrite(&fileheader,1,sizeof(fileheader),fdladder);
@@ -784,10 +723,7 @@ extern int d2ladder_saveladder(void)
 		number=bn_int_get(lhead[i].number);
 		if(number<=0) continue;
 		d2ladder=d2ladderlist_find_type(i);
-		if (!(ldata=malloc(number * sizeof(*ldata)))) {
-			eventlog(eventlog_level_error,__FUNCTION__,"error allocate memory for ldata");
-			continue;
-		}
+		ldata=xmalloc(number * sizeof(*ldata));
 		memset(ldata,0,number * sizeof(*ldata));
 		for (j=0; j< number; j++) {
 			bn_int_set(&ldata[j].experience,d2ladder->info[j].experience);
@@ -797,7 +733,7 @@ extern int d2ladder_saveladder(void)
 			strncpy(ldata[j].charname,d2ladder->info[j].charname,sizeof(ldata[j].charname));
 		}
 		fwrite(ldata,1,number*sizeof(*ldata),fdladder);
-		free(ldata);
+		xfree(ldata);
 	}
 	fclose(fdladder);
 	d2ladder_checksum_set();
@@ -912,11 +848,7 @@ int d2ladder_checksum_set(void)
 		fclose(fdladder);
 		return -1;
 	}
-	if (!(buffer=malloc(filesize))) {
-		fclose(fdladder);
-		eventlog(eventlog_level_error,__FUNCTION__,"could not allocate memory for buffer");
-		return -1;
-	}
+	buffer=xmalloc(filesize);
 
 	curlen=0;
 	while(curlen<filesize) {
@@ -926,9 +858,9 @@ int d2ladder_checksum_set(void)
 		    len = filesize-curlen;
 		readlen=fread(buffer+curlen,1,len,fdladder);
 		if (readlen<=0) {
-			free(buffer);
+			xfree(buffer);
 			fclose(fdladder);
-			eventlog(eventlog_level_error,__FUNCTION__,"got bad save file or read error(read:%s)",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"got bad save file or read error(read:%s)",pstrerror(errno));
 			return -1;
 		}
 		curlen+=readlen;
@@ -937,7 +869,7 @@ int d2ladder_checksum_set(void)
 	bn_int_set(&checksum,d2ladder_checksum(buffer,filesize,LADDERFILE_CHECKSUM_OFFSET));
 	fseek(fdladder,LADDERFILE_CHECKSUM_OFFSET,SEEK_SET);
 	fwrite(&checksum,1,sizeof(checksum),fdladder);
-	free(buffer);
+	xfree(buffer);
 	fclose(fdladder);
 	return 0;
 }
@@ -970,11 +902,7 @@ int d2ladder_checksum_check(void)
 		fclose(fdladder);
 		return -1;
 	}
-	if (!(buffer=malloc(filesize))) {
-		fclose(fdladder);
-		eventlog(eventlog_level_error,__FUNCTION__,"could not allocate memory for buffer");
-		return -1;
-	}
+	buffer=xmalloc(filesize);
 	header=(t_d2ladderfile_header *)buffer;
 	curlen=0;
 	while(curlen<filesize) {
@@ -984,9 +912,9 @@ int d2ladder_checksum_check(void)
 		    len = filesize-curlen;
 		readlen=fread(buffer+curlen,1,len,fdladder);
 		if (readlen<=0) {
-			free(buffer);
+			xfree(buffer);
 			fclose(fdladder);
-			eventlog(eventlog_level_error,__FUNCTION__,"got bad save file or read error(read:%s)",strerror(errno));
+			eventlog(eventlog_level_error,__FUNCTION__,"got bad save file or read error(read:%s)",pstrerror(errno));
 			return -1;
 		}
 		curlen+=readlen;
@@ -995,7 +923,7 @@ int d2ladder_checksum_check(void)
 
 	oldchecksum=bn_int_get(header->checksum);
 	checksum=d2ladder_checksum(buffer,filesize,LADDERFILE_CHECKSUM_OFFSET);
-	free(buffer);
+	xfree(buffer);
 
 	if(oldchecksum==checksum) {
 		eventlog(eventlog_level_info,__FUNCTION__,"ladder file check pass (checksum=0x%X)",checksum);
