@@ -14,6 +14,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include "common/setup_before.h"
+#ifdef WIN32
+
 #include <windows.h>
 #include <winsvc.h>
 #include <stdlib.h>
@@ -234,3 +237,5 @@ void Win32_ServiceRun()
 	{
 	} 
 }
+
+#endif
