@@ -14,6 +14,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifdef WIN32
+
 void Win32_ServiceInstall(void);
 void Win32_ServiceUninstall(void);
 void Win32_ServiceRun(void);
+
+#endif
